/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
//--------------------------------------------------------------------------------
//---- test case: Bandwidth on complete graphs
//--------------------------------------------------------------------------------

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

double ObjectiveFunction::power =8.0;

//--------------------------------------------------------------------------------


double ObjectiveFunction::cost(Permutation & sol)
{
  int *targets, it, i;
  double span, sum=0;
  int size=sol.get_permutation_size();

  for(i=0, it=sol.first_index(); i<size; i++, it=sol.next_index(it)){
    int v = sol[it];
    targets = search_space->in_edges(v);
    for(int walk =0; targets[walk] != NoSuchEdge; ++walk){
      if(sol.having_element(targets[walk])){
	span = abs(it - sol.index_of(targets[walk]));
	sum += pow(span, power);
      }
    }
    //    cout<<"it sum "<<it<<" "<<" "<<end<<" "<<sum<<endl;
  }
  return sum;
}

//--------------------------------------------------------------------------------
  
double ObjectiveFunction::extend_cost(Permutation & sol,
				      const mutation_element &mut_el)
{
  int pos = mut_el.first, new_ele = mut_el.second;
  int *targets, *sources;
  double sum = 0, span;

  if(pos < sol.get_permutation_size() * 0.5){
    for(int it=sol.first_index(); it<pos; it=sol.next_index(it)){
      int v = sol[it];
      targets = search_space->in_edges(v);
      for(int walk =0; targets[walk] != NoSuchEdge; ++walk){
	if(sol.having_element(targets[walk]) && sol.index_of(targets[walk])>=pos){
	  span = sol.index_of(targets[walk]) - it;
	  sum += (pow(span+1.0, power) - pow(span, power));
	}
      }
      sources = search_space->out_edges(v);
      for(int walk =0; sources[walk] != NoSuchEdge; ++walk){
	if(sol.having_element(sources[walk]) && sol.index_of(sources[walk])>=pos){
	span = sol.index_of(sources[walk]) - it;
	sum += (pow(span+1.0, power) - pow(span, power));
	}
      }
    }
  }
  else {
    for(int it=pos; it<=sol.last_index(); it=sol.next_index(it)){
      int v = sol[it];
      targets = search_space->in_edges(v);
      for(int walk =0; targets[walk] != NoSuchEdge; ++walk){
	if(sol.having_element(targets[walk]) && sol.index_of(targets[walk])<pos){
	  span = it - sol.index_of(targets[walk]);
	  sum += (pow(span+1.0, power) - pow(span, power));
	}
      }
      sources = search_space->out_edges(v);
      for(int walk =0; sources[walk] != NoSuchEdge; ++walk){
	if(sol.having_element(sources[walk]) && sol.index_of(sources[walk])<pos){
	  span = it - sol.index_of(sources[walk]);
	  sum += (pow(span+1.0, power) - pow(span, power));
	}
      }
    }
  }

  ///////////=====================

  targets = search_space->in_edges(new_ele);
  sources = search_space->out_edges(new_ele);
  
  for(int walk=0; targets[walk] != NoSuchEdge; ++walk){
    if(sol.having_element(targets[walk])){
      if(sol.index_of(targets[walk]) >= pos)
	span = sol.index_of(targets[walk]) - pos + 1;
      else span = pos - sol.index_of(targets[walk]);

      sum += pow(span, power);
    }
  }

  for(int walk=0; sources[walk] != NoSuchEdge; ++walk){
    if(sol.having_element(sources[walk])){
      if(sol.index_of(sources[walk]) >= pos)
	span = sol.index_of(sources[walk]) - pos + 1;
      else span = pos - sol.index_of(sources[walk]);

      sum += pow(span, power);
    }
  }
 
  return sum;
}

//--------------------------------------------------------------------------------


double ObjectiveFunction::delta_cost(Permutation & sol,
				     const mutation_element & mut_el)
{
  int *targets, *sources;
  int first, second, first_element, second_element;
  double sum_before=0, sum_after=0;

  first = mut_el.first;
  second = mut_el.second;
  first_element = sol[first];
  second_element = sol[second];
  
  targets = search_space->in_edges(first_element);
  sources = search_space->out_edges(first_element);

  for(int walk=0; targets[walk] != NoSuchEdge; ++walk){
    if(targets[walk] != second_element && sol.having_element(targets[walk])){
      sum_before += pow(abs(sol.index_of(targets[walk])-first), power);
      sum_after += pow(abs(sol.index_of(targets[walk])-second), power);
    }
  }

  for(int walk=0; sources[walk] != NoSuchEdge; ++walk){
    if(sources[walk] != second_element && sol.having_element(sources[walk])){
      sum_before += pow(abs(sol.index_of(sources[walk])-first), power);
      sum_after += pow(abs(sol.index_of(sources[walk])-second), power);
    }
  }
  
  targets = search_space->in_edges(second_element);
  sources = search_space->out_edges(second_element);
  
  for(int walk=0; targets[walk] != NoSuchEdge; ++walk){
    if(targets[walk] != first_element && sol.having_element(targets[walk])){
      sum_before += pow(abs(sol.index_of(targets[walk])-second), power);
      sum_after += pow(abs(sol.index_of(targets[walk])-first), power);
    }
  }
  for(int walk=0; sources[walk] != NoSuchEdge; ++walk){
    if(sources[walk] != first_element && sol.having_element(sources[walk])){
      sum_before += pow(abs(sol.index_of(sources[walk])-second), power);
      sum_after += pow(abs(sol.index_of(sources[walk])-first), power);
    }
  }
  
  return sum_after - sum_before;
}
//--------------------------------------------------------------------------------
  
double ObjectiveFunction::correctness(Permutation & sol)
{
  return 0;
}

  
double ObjectiveFunction::delta_correctness(Permutation & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

  
double ObjectiveFunction::extend_correctness(Permutation & sol,
					     const mutation_element &mut_el)
{
  return 0;
}

//--------------------------------------------------------------------------------


double ObjectiveFunction::true_cost(Permutation & sol)
{
  int *targets, span, biggest_span = 0, it, i;
  int size = sol.get_permutation_size();

  for(i=0, it=sol.first_index(); i<size; i++, it=sol.next_index(it)){
    int v = sol[it];
    targets = search_space->in_edges(v);
    for(int walk =0; targets[walk] != NoSuchEdge; ++walk){
      if(sol.having_element(targets[walk])){
	span = abs(it - sol.index_of(targets[walk]));
	if(span > biggest_span) biggest_span = span;
      }
    }
  }
  return biggest_span;
}
//--------------------------------------------------------------------------------


double ObjectiveFunction::true_correctness(Permutation & sol)
{
  return correctness(sol);
}

//--------------------------------------------------------------------------------
