/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
#include "unweighted_adjacency_list.h"

using std::ofstream;
using std::cout;
using std::endl;

int Graph::size = 0;

int Graph::get_size()
{
  return size;
}


Graph::Graph(char *GML_file_name)
{
	graph G;
	graph::node_iterator node_it;
	graph::node_iterator node_end;
	node::in_edges_iterator in_it;
	node::in_edges_iterator in_end;
	node::out_edges_iterator out_it;
	node::out_edges_iterator out_end;

  if(G.load(GML_file_name).err_num != GML_OK){
    cout << "could not open file " << GML_file_name << " to read graph\n";
    exit(1);
  }

  this->num_of_nodes = G.number_of_nodes();
  this->num_of_edges = G.number_of_edges();
  this->in_list = new (int*)[num_of_nodes];
  this->out_list = new (int*)[num_of_nodes];
  
  node_it = G.nodes_begin();
  node_end = G.nodes_end();
  for(int i=num_of_nodes-1; node_it != node_end; ++node_it, --i){
    this->node_index[*node_it] = i+1;
    in_list[i] = new int[node_it->indeg()+1];
    in_list[i][node_it->indeg()] = NoSuchEdge;
    out_list[i] = new int[node_it->outdeg()+1];
    out_list[i][node_it->outdeg()] = NoSuchEdge;
  }
  
  node_it = G.nodes_begin();
  node_end = G.nodes_end();
  for(; node_it != node_end; ++node_it){
    in_it = (*node_it).in_edges_begin();
    in_end = (*node_it).in_edges_end();
    for(int j=0; in_it != in_end; ++j, ++in_it)
      in_list[node_index[*node_it]-1][j] = this->node_index[(*in_it).source()];
    out_it = (*node_it).out_edges_begin();
    out_end = (*node_it).out_edges_end();
    for(int j=0; out_it != out_end; ++j, ++out_it)
      out_list[node_index[*node_it]-1][j] =
	this->node_index[(*out_it).target()];
  }
  
  size = num_of_nodes;
}

int * Graph::in_edges(int x)
{
	return in_list[x-1];
}

int * Graph::out_edges(int x)
{
	return out_list[x-1];
}

Graph::~Graph()
{
	delete [] in_list;
	delete [] out_list;
}

void Graph::display()
{
	for(int i=0; i < num_of_nodes; ++i){
		cout << endl << i+1;
		for(int j=0; out_list[i][j] != NoSuchEdge; ++j)
			cout << "->" << out_list[i][j];
	}
	cout << endl;
			
}

int Graph::get_num_of_edges()
{ 
	return num_of_edges;
}

bool Graph::edge_exists(int x, int y)
{
	--x;
	--y;
	for(int i=0; out_list[x][i] != NoSuchEdge; ++i)
		if(out_list[x][i] == y) return true;
	return false;
}
