/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#include "edge_graph.h"
#include <fstream>
using std::ifstream;

//--------------------------------------------------------------------
// DIMACS Format
//--------------------------------------------------------------------
int EdgeGraph::size = 0;

int EdgeGraph::get_size() { return size; }


EdgeGraph::EdgeGraph(char *file_name)
{
  //----------------------------------------//
  // This is for DIMACS problems (coloring on undirected graphs) only

  the_edge = NULL;
  edges = vertices = 0;
  ifstream in(file_name);
  if(!in){
    cerr << "can't open "<<file_name<<endl;
    exit(0);
  }
  char description[LINE_SIZE+1];
  int i;

  in.getline(description, LINE_SIZE);
  //  while(description[0] != '\0'){
  while(in){
    switch(parse(description)){

    case comment:
      break;

    case edge_problem:
      for(i=2; description[i]!=' '; i++);
      vertices = atoi(&description[i]);

      // the_edge[i] is the list of neighbors of vertex i
      the_edge = new edge_set [vertices + 1];

      for(i=i+1; description[i]!=' '; i++);
      edges = atoi(&description[i]);
      break;

    case edge:
      Edge a,b;
      a.vertex = atoi(&description[1]);
      for(i=2; description[i]!=' '; i++);
      b.vertex = atoi(&description[i]);
      for(i++; description[i]!=' ' && description[i]!='\0'; i++);
      a.weight = b.weight = atoi(&description[i]);

      the_edge[a.vertex].insert(b);
      the_edge[b.vertex].insert(a);

      //      cout<<a.vertex<<" "<<b.vertex<<" "<<a.weight<<endl;
      break;

      //    default:
      //      cerr << "EdgeGraph::EdgeGraph: unknown type\n";
      //      exit(0);
    }
    in.getline(description, LINE_SIZE);
  }

  in.close();
  size = vertices;

  //cout << *this << endl;  exit(0);
}

//--------------------------------------------------------------------
EdgeGraph::~EdgeGraph()
{
  if(the_edge != NULL) delete [] the_edge;
}

//--------------------------------------------------------------------

TokenType EdgeGraph::parse(char *description)
{
  switch(description[0]){
  case 'c': return comment;
  case 'p':
    if(description[2] == 'e' && description[3] == 'd' && 
       description[4] == 'g' && description[5] == 'e')
      return edge_problem;
    else return unknown;
  case 'e': return edge;
  case 'n': return node;
  }
  return unknown;
}

//--------------------------------------------------------------------
bool EdgeGraph::edge_exist(int source, int dest) const
{
  if(source<1 || source>vertices) non_exist_error(source, "edge_exist()");
  if(dest<1 || dest>vertices) non_exist_error(dest, "edge_exist()");
  Edge e;
  e.vertex = dest;
  edge_set::iterator it = the_edge[source].find(e);
  return (it != the_edge[source].end());
}
//--------------------------------------------------------------------
edge_set * EdgeGraph::get_edges(int v)
{
  if(v<1 || v>vertices) non_exist_error(v, "get_edges()");
  return &(the_edge[v]);
}

//--------------------------------------------------------------------
int EdgeGraph::degree(int v) const
{
  if(v<1 || v>vertices) non_exist_error(v, "degree()");
  return (int) the_edge[v].size();
}
//--------------------------------------------------------------------
ostream & operator<<(ostream & out, EdgeGraph & g)
{
  int i;
  out << g.vertices <<" vertices, "<<g.edges<< " edges\n";
  for(i=1; i<=g.vertices; i++){
    out << "edge "<< i << " (d = "<<g.degree(i)<<") --> ";
    edge_set::iterator e = g.the_edge[i].begin();
    for( ; e!=g.the_edge[i].end(); e++) out << e->vertex << " ";
    out << endl;
  }
  return out;
}
//--------------------------------------------------------------------
bool EdgeGraph::remove_edge(int source, int dest) 
{
  if(source<1 || source>vertices) non_exist_error(source, "remove_edge()");
  if(dest<1 || dest>vertices) non_exist_error(dest, "remove_edge()");
  Edge e1, e2;
  e1.vertex = dest;
  e2.vertex = source;
  edge_set::iterator it1 = the_edge[source].find(e1), it2 = the_edge[dest].find(e2);
  if(it1 == the_edge[source].end() || it2 == the_edge[dest].end()) return false;
  the_edge[source].erase(*it1);
  the_edge[dest].erase(*it2);
  edges--;
  return true;
}
//--------------------------------------------------------------------
bool EdgeGraph::remove_edges_containing(int v)
{
  if(v<1 || v>vertices) non_exist_error(v, "remove_edges_containing()");
  edge_set::iterator it, begin=the_edge[v].begin(), end=the_edge[v].end();
  for(it=begin; it!=end; it++){
    Edge e;
    e.vertex = v;
    the_edge[it->vertex].erase(e);
  }
  the_edge[v].erase(the_edge[v].begin(), the_edge[v].end());
  return true;
}
//--------------------------------------------------------------------
