/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
//--------------------------------------------------------------------
// @author: Vinhthuy Phan, 04/2002
//
// -- semi-adjacency list:
//        node access in O(1) time (array)
//        edge access in O(log n) time (STL std::set, red/black tree)  
// -- in_edge[1..n]
//        in_edge[i] : the set of all edges (i, u)
// -- out_edge[1..]
//        out_edge[i] : the set of all edges (u, i)
//
// Vertex indices start from 1.
//--------------------------------------------------------------------

#ifndef EDGE_GRAPH_H
#define EDGE_GRAPH_H

#include <iostream>
#include <set>
#include <functional>
#include <string.h>

using std::set;
using std::binary_function;
using std::ostream;
using std::cout;
using std::endl;
using std::cerr;

//--------------------------------------------------------------------
typedef struct my_edge
{
  int vertex;  // destination 
  int weight;  // edge weight
} Edge;

//--------------------------------------------------------------------

struct EdgeCompare : public binary_function<Edge, Edge, bool>
{
  bool operator()(const Edge & a, const Edge & b) const
  {
    return (a.vertex < b.vertex);
  }
};

typedef set< Edge, EdgeCompare > edge_set;

const int LINE_SIZE = 200;


enum TokenType { comment, edge_problem, edge, node, unknown };

/// --------------------------------------------------------------------
/**
@author Vinhthuy Phan
@version 1.0
*/
class EdgeGraph
{
  ///
  friend ostream & operator<<(ostream & out, EdgeGraph & g);

private:
  ///
  TokenType parse(char *description);

public:
  ///
  edge_set *the_edge;
  ///
  int vertices, edges;
  ///
  EdgeGraph(char *file_name);
  ///
  ~EdgeGraph();
  ///
  static int size;
  ///  
  static int get_size () ;
  ///
  int num_of_vertices () const { return vertices; }
  ///
  int num_of_edges () const { return edges; }
  ///
  int degree(int v) const;
  ///
  bool edge_exist(int source, int dest) const;
  ///
  edge_set * get_edges(int v) ;
  ///
  void non_exist_error(int v, char* where="") const
  {
    cerr << "EdgeGraph::"<<where<<" "<<v << " does not exist\n";
    exit(0);
  }
  ///
  int first_vertex() const { return 1; }
  ///
  int last_vertex() const { return vertices; }
  ///
  bool remove_edge(int source, int dest) ;
  ///
  bool remove_edges_containing(int v);

};
/// --------------------------------------------------------------------

#endif
