/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

// Johnson original function

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

const double penalty_const = 2.0;

//--------------------------------------------------------------------------------
  
double ObjectiveFunction::cost(Partition & sol)
{
  double sum = 0;
  int first = sol.first_part();
  int last = sol.last_part(), i;
  subset same_color_set;

  for(i=first; i<=last; i++){
    same_color_set = sol.get_part(i);
	subset::iterator p, q;
    subset::iterator beg = same_color_set.begin(), end = same_color_set.end();

    int bad_edges = 0, size = (int) same_color_set.size();

    for(p=beg; p!=end; ++p){
      q = p;
      for(q++; q!=end; q++){
	if(*p != *q && search_space->edge_exist(*p, *q))
	  bad_edges++;
      }
    }

    sum += size * (penalty_const * bad_edges - size);
  }

  return sum;
}

//--------------------------------------------------------------------------------
  
double ObjectiveFunction::extend_cost(Partition & sol,
				      const mutation_element &mut_el)
{
  int which_part = mut_el.first;
  if(which_part > sol.num_of_parts()) return -1;

  double old_bad_edges = 0, new_bad_edges = 0;
  int new_ele = mut_el.second;
  int size = sol.partition_size(which_part);
  subset color_set = sol.get_part(which_part);
  subset::const_iterator beg = color_set.begin(), end = color_set.end(), p, q;

  for(p = beg; p!= end; ++p){
    q = p;
    for(q++; q!=end; q++){
      if(search_space->edge_exist(*p, *q))
	old_bad_edges += 1.0;
    }
    if(search_space->edge_exist(new_ele, *p)) new_bad_edges += 1.0;
  }

  //  double new_cost = -2.0*size - 1.0 + 2.0*old_bad_edges + 
  //    2.0*new_bad_edges*(size + 1.0);

  double before = size*(penalty_const * old_bad_edges - size);
  double after = (size+1.0)*(penalty_const * (old_bad_edges+new_bad_edges) - (size+1.0));

  return after - before;
}
//--------------------------------------------------------------------------------
// return the change in cost of mutating sol
//--------------------------------------------------------------------------------
  
double ObjectiveFunction::delta_cost(Partition & sol,
				     const mutation_element &mut_el)
{
  int rand_ele = mut_el.first;
  int old_color = sol.which_part(rand_ele);
  int new_color = mut_el.second;

  int total_old = 0, total_new = 0, old_bad_edges = 0, new_bad_edges = 0;

  subset old_color_set, new_color_set;
  subset::const_iterator p, q, beg, end;

  old_color_set = sol.get_part(old_color);
  beg = old_color_set.begin(), end = old_color_set.end();
  for(p=beg; p!=end; ++p){
    q = p;
    for(q++; q!=end; q++){
      if(*p != *q && search_space->edge_exist(*p, *q)){
	total_old++;
	if(*p == rand_ele || *q == rand_ele)
	  old_bad_edges++;
      }
    }
  }

  new_color_set = sol.get_part(new_color);
  beg = new_color_set.begin(), end = new_color_set.end();
  for(p=beg; p!=end; ++p){
    q = p;
    for(q++; q!=end; q++){
      if(*p != *q && search_space->edge_exist(*p, *q)){
	total_new++;
      }
    }
    if(search_space->edge_exist(*p, rand_ele)){
      new_bad_edges++;
      //      new_bad_edges++;
    }
    //    if(search_space->edge_exist(rand_ele, *p))      new_bad_edges++;
  }

  //  cout<<"new_bad_edges :"<<new_bad_edges<<endl;

  int old_size = (int)old_color_set.size(), new_size = (int)new_color_set.size();

  double before = old_size*(penalty_const*total_old - old_size) 
    + new_size*(penalty_const*total_new - new_size);

  double after = (old_size-1.0)*(penalty_const*(total_old - old_bad_edges) - (old_size-1.0)) +
                 (new_size+1.0)*(penalty_const*(total_new + new_bad_edges) - (new_size+1.0));
  return after - before;
}

//--------------------------------------------------------------------------------
double ObjectiveFunction::correctness(Partition & sol)
{
  return 0;
}

double ObjectiveFunction::delta_correctness(Partition & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

double ObjectiveFunction::extend_correctness(Partition & sol,
					     const mutation_element &mut_el)
{
  return 0;
}
//--------------------------------------------------------------------------------

double ObjectiveFunction::true_cost(Partition & sol)
{
  return sol.num_of_parts();
}

//--------------------------------------------------------------------------------

double ObjectiveFunction::true_correctness(Partition & sol)
{
  int violation = 0, i, last_vertex = search_space->last_vertex();
  edge_set * E;

  for(i= search_space->first_vertex(); i<=last_vertex; i++){
    E = search_space->get_edges(i);

    edge_set::iterator e, last_edge = E->end();

    if(sol.having_element(i)){
      int vertex_color = sol.which_part(i);
      
      for(e = E->begin(); e!=last_edge; e++){
	if(e->vertex < i && sol.having_element(e->vertex)){
	  int neighbor_color = sol.which_part(e->vertex);

	  if(vertex_color == neighbor_color)
	    violation++;
	}
      }
    }
  }

  return violation;
  //  return 0.5 * violation;
}

//--------------------------------------------------------------------------------
