/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#ifndef GREEDY_CPP
#define GREEDY_CPP

#include <math.h>
#include "../basic_search.h"
#include "hill_climb.cpp"

//----------------------------------------------------------------------------------
// If K is underestimated, double it (i.e. if(how_many >= K) K *= 2.0))
// 2 is found to be better than 1.1, 1.5, and 3.0
//----------------------------------------------------------------------------------
/**
heuristic.
@author Vinhthuy Phan
@version 1.0
*/
template <class T>
class Greedy : public BasicSearch<T> {

public:
  ///
  int MaxSize;
  ///
  int best_pos;
  ///
  int best_element;
  ///
  int first_pos;
  ///
  int last_pos;
  ///
  double best_fitness;
  ///
  int *the_element;
  ///
  int *the_position;
  ///
  double K;

  ///
  double *ext_time;
  ///
  double *ext_gen_time;

  ///
  Greedy(double running_time, double r_interval, double cp_interval,
	   Solution<T> * an_init_sol = NULL);
  
  ///
  ~Greedy();

  ///
  bool estimate_params();
  ///
  void mixup_position(int first, int last);
  ///
  void mixup_element();

  ///
  void set_limits(double num_ele_tried, double num_pos, double *element_limit, double *position_limit);

  ///
  bool find_best_extension(Solution<T> & sol, double *how_many);

  ///
  void search();

  ///
  void search_specific_display(ostream *out) { }

  ///
  double overhead_time();

  ///
  void finish_up(Solution<T> * sol);

};

//----------------------------------------------------------------------------------

template<class T>
Greedy<T>::Greedy(double running_time, double r_interval, double cp_interval,
					Solution<T> * an_init_sol)
  : BasicSearch<T>(running_time, r_interval, cp_interval, an_init_sol)
{
  MaxSize = ObjectiveFunction::get_size();

  int i;

  ext_time = new double [MaxSize + 1];
  ext_gen_time = new double [MaxSize + 1];
  the_element = new int [MaxSize + 1];
  the_position = new int [MaxSize + 1];
  for(i=1; i<=MaxSize; i++){
    the_element[i] = i;
    ext_time[i] = ext_gen_time[i] = 0;
  }
  mixup_element();
}

//----------------------------------------------------------------------------------

template<class T>
Greedy<T>::~Greedy()
{
  if(the_element!=NULL)  delete [] the_element;
  if(the_position!=NULL) delete [] the_position;
  if(ext_time!=NULL) delete [] ext_time;
  if(ext_gen_time!=NULL) delete [] ext_gen_time;
}

//----------------------------------------------------------------------------------

template<class T>
double Greedy<T>::overhead_time()
{
  return Global<T>::get_solution_creation_time();
}


//----------------------------------------------------------------------------------

template<class T>
void Greedy<T>::mixup_element()
{
  int i, r, tmp;
  for(i=1; i<MaxSize; i++){
    r = i + (rand() % (MaxSize - i + 1));
    tmp = the_element[i];
    the_element[i] = the_element[r];
    the_element[r] = tmp;
  }
}

//----------------------------------------------------------------------------------

template<class T>
void Greedy<T>::mixup_position(int first, int last)
{
  int i, r, tmp, v, size = last-first+1;
  for(i=1, v=first; v<=last; v++, i++) the_position[i] = v;
  for(i=1; i<size; i++){
    r = i + (rand() % (size - i + 1));
    tmp = the_position[i];
    the_position[i] = the_position[r];
    the_position[r] = tmp;
  }
}

//----------------------------------------------------------------------------------

template<class T>
bool Greedy<T>::estimate_params()
{
#if debug
  cout << "#Greedy<T>::estimate_params()\n";
#endif

#if 0
  Solution<T> sol, tmp_sol;
  gen_random_solution(&sol, 2);
  while(sol.get_num_elements() < MaxSize){
    double t = get_remaining_time();
    int element;
    for(int e=1; e<=MaxSize; e++){
      element = the_element[e];
      if(! sol.object->having_element(element)){
	sol.gen_extension(&tmp_sol, 1, element);
	break;
      }
    }
    ext_gen_time[sol.get_num_elements()] = t - get_remaining_time();
    t = get_remaining_time();
    sol.extend(1, element);
    ext_time[sol.get_num_elements()] = t - get_remaining_time();
  }

  double total_gen_time=0, total_ext_time = 0;
  for(int i=1; i<=MaxSize; i++){
    total_gen_time += ext_gen_time[i];
    total_ext_time += ext_time[i];
  }


  K = (get_remaining_time() - total_ext_time)/total_gen_time;
  //  K = 3;
  //  cout<<"K is "<<K<<"\t"<<total_ext_time<<"\t"<<total_gen_time<<"\t"<<get_remaining_time()<<endl;
  if(K < 1.0) {
    K = 1.0;
  }

#else
  //  K = 3.0;
  K = 4.0;
#endif

  return true;
}


//----------------------------------------------------------------------------------
template<class T>
void Greedy<T>::finish_up(Solution<T> * sol)
{
#if debug
  cout<<"Greedy::finish_up()\n";
#endif

  while(sol->get_num_elements() < MaxSize){
    if(get_remaining_time() <= 0) break;

    for(int e=1; e<=MaxSize; e++){
      int element = the_element[e];
      if(! sol->object->having_element(element)){
	if(! ObjectiveFunction::correctness_is_defined())
	  sol->extend(sol->object->first_position(), element);
	else {
	  first_pos = sol->object->first_position();
	  last_pos = sol->object->last_position();
	  
	  Solution<T> tmp_sol;
	  for(int p=first_pos; p<=last_pos; p++){
	    sol->gen_extension(&tmp_sol, p, element);
	    if(tmp_sol.get_correctness() == 0){
	      sol->extend(p, element);
	      break;
	    }
	  }
	}
      }
    }
  }
}
//----------------------------------------------------------------------------------
template<class T>
void Greedy<T>::set_limits(double num_ele_tried, double num_pos,
			   double *element_limit, double *position_limit)
{
  
  double x = sqrt((double)(num_pos * num_ele_tried)/(double)K);

  if(num_ele_tried >= x && num_pos >= x){
    *element_limit = num_ele_tried / x;
    *position_limit = num_pos / x;
    if(*element_limit > num_ele_tried) *element_limit = num_ele_tried;
    if(*position_limit > num_pos) *position_limit = num_pos;
    //    cout<<"#1: "<<*element_limit * *position_limit<<"  "<<K<<" "<<num_ele_tried * num_pos<<" "<<x<<endl;
  }
  else if(num_ele_tried < x && num_pos >= x){
    *element_limit = num_ele_tried * 0.5;
    if(*element_limit < 1) *element_limit = 1;
    *position_limit = (double)K / *element_limit;
    if(*position_limit > num_pos) *position_limit = num_pos;
  }
  else if(num_ele_tried >= x && num_pos < x){
    *position_limit = num_pos * 0.5;
    if(*position_limit < 1) *position_limit = 1;
    *element_limit = (double)K / *position_limit;
    if(*element_limit > num_ele_tried) *element_limit = num_ele_tried;
  }
  else if(num_ele_tried < x && num_pos < x){
  //    cout<<"#Greedy::search (2): not enough time "<<num_ele_tried<<" "<<num_pos<<" "<<x<<" "<<K<<endl;
  //    return false;
    *position_limit = 1;
    *element_limit = 1;
  }

}

//----------------------------------------------------------------------------------
template<class T>
bool Greedy<T>::find_best_extension(Solution<T> & sol, double *how_many)
{
  Solution<T> tmp_sol;
  double num_ele_tried = MaxSize - sol.get_num_elements();
  double position_limit, element_limit, num_pos;
  bool at_least_one_chosen = false;
  
  first_pos = sol.object->first_position();
  last_pos = sol.object->last_position();
  num_pos = last_pos - first_pos + 1;
  
  *how_many = num_pos * num_ele_tried;
  mixup_position(first_pos, last_pos);
  set_limits(num_ele_tried, num_pos, &element_limit, &position_limit);
  
  int e, count;
  double best_ext_correctness;
  for(e=1, count=0; count<element_limit; e++){
    int element = the_element[e];
    
    if(! sol.object->having_element(element)){
      count++;
      
      // -- Test positions
      for(int p=1, c=0; c<position_limit; p++, c++){
	int pos = the_position[p];
	sol.gen_extension(&tmp_sol, pos, element);
	
	if((tmp_sol.get_fitness() < best_fitness) || !at_least_one_chosen){
	  best_pos = pos;
	  best_element = element;
	  best_fitness = tmp_sol.get_fitness();
	  best_ext_correctness = tmp_sol.get_correctness();
	  at_least_one_chosen = true;
	}
      }
    }
  }

  if(best_ext_correctness > 0){  
    // if the best extension is not correct, make sure it is
    for(int i=first_pos; i<=last_pos; i++){
      sol.gen_extension(&tmp_sol, i, best_element);
      if(tmp_sol.get_correctness() == 0){
	best_pos = i;
	best_ext_correctness = 0;
	break;
      }
    }
    if(best_ext_correctness > 0) return false;
    //    cout<<"best new pos is "<<best_pos<<endl;
  }

  //  cout <<"\t#: "<<sol.get_cost()<<" "<<sol.get_true_cost()<<endl;

#if 0
  cout <<"\t#: "<<sol.get_cost()<<" "<<sol.get_correctness()<<" "<<sol.get_true_correctness()
       <<"  "<<sol.get_num_elements()<<"  "<<element_limit<<" K="<<K
       <<"  "<<num_ele_tried<<"\t"<<get_remaining_time()
       <<"  "<<best_pos<<" "<<best_element<<" "<<best_ext_correctness<<endl;
#endif
  
  return at_least_one_chosen;
}
//----------------------------------------------------------------------------------
template<class T>
void Greedy<T>::search()
{
  if(search_stat == stat_terminate) return;
  ObjectiveFunction::complete_solution_flag = false;

  if(estimate_params() == false) return;
  Solution<T> sol;
  gen_random_solution(&sol, 2);

  // -- N times (N max size of solution)
  while(sol.get_num_elements() < MaxSize){
    double time_before = get_remaining_time(), time_after;
    double how_many=0;

    if(find_best_extension(sol, &how_many) == true){
      sol.extend(best_pos, best_element);
    }
    else {
      //      cout<<sol.get_num_elements()<<endl;
      //      exit(0);
    }

    if(get_remaining_time() <= 0) break;


    time_after = get_remaining_time();
    double loop_time = (time_before - time_after);
    double dt = time_after/(double)(MaxSize - sol.get_num_elements());
    
#if 1
    if(loop_time < 0.5 * dt){ // more than enough time
      if(how_many >= K) K *= 2.0;  // to avoid K goes to infinity
    }
    else K = K * dt/loop_time;
#endif
#if 0
    cout <<"\t#: "<<sol.get_cost()<<" "<<sol.get_correctness()//<<" "<<sol.get_true_correctness()
	 <<"  "<<sol.get_num_elements()<<"  "<<how_many<<" K="<<K<<"\t"<<get_remaining_time()
	 <<"  "<<best_pos<<" "<<best_element<<" "<<loop_time<<" "<<dt<<endl;
#endif
  }



#if debug
  cout<<"remaining time: "<<get_remaining_time()<<endl;
  cout<<"solution size is: "<<sol.get_num_elements()<<", (MaxSize is "<<MaxSize<<")\n";
  cout<<"true cost: "<<sol.get_true_cost()<<endl;
  cout<<"true correctness: "<<sol.get_true_correctness()<<endl;
#endif

  finish_up(&sol);
  
  current_solution = sol;
  update_best_solution(current_solution);
  //  cout<<"remaining time: "<<get_remaining_time()<<endl;
  //  cout<<current_solution.object<<" "<<best_solution.object<<" "<<best_solution.get_fitness()<<" "
  //      <<Global<T>::best_solution.get_fitness()<<" "<<Global<T>::best_solution.object<<endl;
}


//----------------------------------------------------------------------------------
#endif
