/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

//--------------------------------------------------------------------------------

double ObjectiveFunction::cost(Subset & sol)
{
  double span = 0;
  subset * setA = sol.get_subset();
  int target_index, source_index;
  node target, source;

  for(subset::iterator el = setA->begin(); el != setA->end(); ++el){
    node the_element = search_space->node_from_index(*el);

    for(node::out_edges_iterator it_out = the_element.out_edges_begin();
	it_out != the_element.out_edges_end(); ++it_out){
      target = it_out->target();
      target_index = search_space->index_from_node(target);
      if(sol.having_element(target_index) && !sol.contains(target_index)){
	span += 1.0;
      }
    }
    
    for(node::in_edges_iterator it_in = the_element.in_edges_begin();
	it_in != the_element.in_edges_end(); ++it_in){
      source = it_in->source();
      source_index = search_space->index_from_node(source);
      if(sol.having_element(source_index) && !sol.contains(source_index)){
	span += 1.0;
      }
    }
  }

#if 0
  for(graph::edge_iterator it = search_space->edges_begin();
      it != search_space->edges_end(); ++it){
    node source = it->source();
    node target = it->target();
    int source_index = search_space->index_from_node(source);
    int target_index = search_space->index_from_node(target);
    if(complete_solution_flag ||
       (sol.having_element(source_index) && sol.having_element(target_index))){
      bool set_of_source = sol.contains(source_index);
      bool set_of_target = sol.contains(target_index);
      if(set_of_source != set_of_target) ++span;
    }
  }
#endif
  return (search_space->number_of_edges() - span);
}

//--------------------------------------------------------------------------------

double ObjectiveFunction::extend_cost(Subset & sol, const mutation_element & mut_el)
{
  double span=0;
  int target_index, source_index;
  bool set_of_vertex;
  node target, source;
  node new_ele = search_space->node_from_index(mut_el.second);

  if(mut_el.first == 1) set_of_vertex = true;
  else set_of_vertex = false;

  for(node::out_edges_iterator it_out = new_ele.out_edges_begin();
      it_out != new_ele.out_edges_end(); ++it_out){
    target = it_out->target();
    target_index = search_space->index_from_node(target);
    if(sol.having_element(target_index) && sol.contains(target_index) != set_of_vertex){
      span += 1.0;
      //      cout<<"out: "<<set_of_vertex<<" "<<target_index<<endl;
    }
  }
  
  for(node::in_edges_iterator it_in = new_ele.in_edges_begin();
      it_in != new_ele.in_edges_end(); ++it_in){
    source = it_in->source();
    source_index = search_space->index_from_node(source);
    if(sol.having_element(source_index) && sol.contains(source_index) != set_of_vertex){
      span += 1.0;
      //      cout<<"in: "<<set_of_vertex<<" "<<source_index<<endl;
    }
  }
  //  cout<<"span is "<<span<<endl;
  return - span;
}
//--------------------------------------------------------------------------------

double ObjectiveFunction::delta_cost(Subset & sol,
				     const mutation_element & mut_el)
{
  double new_span = 0;
  node target, source;
  int target_index, source_index;

  bool set_of_vertex = sol.contains(mut_el.first);
  node swap_vertex = search_space->node_from_index(mut_el.first);
  
  
  for(node::out_edges_iterator it_out = swap_vertex.out_edges_begin();
      it_out != swap_vertex.out_edges_end(); ++it_out){
    target = it_out->target();
    target_index = search_space->index_from_node(target);
    if(complete_solution_flag || sol.having_element(target_index)){
      if(sol.contains(target_index) == set_of_vertex)
	new_span += 1.0;
    }
  }
  
  for(node::in_edges_iterator it_in = swap_vertex.in_edges_begin();
      it_in != swap_vertex.in_edges_end(); ++it_in){
    source = it_in->source();
    source_index = search_space->index_from_node(source);
    if(complete_solution_flag || sol.having_element(source_index)){
      if(sol.contains(source_index) == set_of_vertex)
	new_span += 1.0;
    }
  }
  
  double old_span = swap_vertex.degree() - new_span;
  //  cout<<old_span-new_span<<endl;
  return ( old_span - new_span );
}
//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
  
double ObjectiveFunction::correctness(Subset & sol)
{
  return 0;
}

  
double ObjectiveFunction::delta_correctness(Subset & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

  
double ObjectiveFunction::extend_correctness(Subset & sol,
					     const mutation_element &mut_el)
{
  return 0;
}
//--------------------------------------------------------------------------------

double ObjectiveFunction::true_cost(Subset & sol)
{
  return cost(sol);
}
  

double ObjectiveFunction::true_correctness(Subset & sol)
{
  return correctness(sol);
}
//--------------------------------------------------------------------------------
