/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/

#include "maxcut_graphs.h"


int weighted_graph::size = 0;

int weighted_graph::get_size() { return size; }

weighted_graph::weighted_graph() : graph()
{
}

weighted_graph::weighted_graph(const char *file_name) : graph()
{
  if(this->load(file_name).err_num != GML_OK) error_call(bad_gml_file);

  size = this->number_of_nodes();
}

///////////////////////////////////////////////////////////////////////
weighted_graph::~weighted_graph()
{
  delete [] indexed_nodes;
}

///////////////////////////////////////////////////////////////////////
void weighted_graph::load_edge_info_handler (edge e, GML_pair* list)
{

  if(list == NULL){
    cout <<endl << "Missing weights!!!" << endl;
    return;
  }
  
  edge_handler eh;
  
  if(list->kind == GML_DOUBLE) eh.w = list->value.floating;
  else if(list->kind == GML_INT) eh.w = list->value.integer;
  else{
    cout << "Bad weights!!!" << endl;
    return;
  }
  
  eh.e = e;

  weight_vector.push_back(eh);
}


///////////////////////////////////////////////////////////////////////
void weighted_graph::load_graph_info_handler(GML_pair* list){

  indexed_nodes = new node [this->number_of_nodes() + 1];

  graph::node_iterator it = this->nodes_begin();
  graph::node_iterator end = this->nodes_end();

  for(int index=1 ; it != end; ++it, ++index){
    indexed_nodes[index] = *it;
    node_index_map[*it] = index;
  }

  //We initialize the weight to be one for all edges
  std::vector<edge_handler>::const_iterator weight_iter = weight_vector.begin();
  std::vector<edge_handler>::const_iterator weight_iter_end =
		weight_vector.end();
 
  for( ; weight_iter != weight_iter_end; ++weight_iter){
    node x = weight_iter->e.source();
    node y = weight_iter->e.target();
    
    node::out_edges_iterator edge_it = x.out_edges_begin();
    node::out_edges_iterator edge_it_end = x.out_edges_end();
    
    bool flag = false;
    
    for(; edge_it != edge_it_end; ++ edge_it){
      if( edge_it->target() == y) weight[*edge_it] = weight_iter->w;
      flag = true;
    }
    
    if(!flag) cout << "We have a weight with no initialization!!!\n";

  }
}


///////////////////////////////////////////////////////////////////////
double weighted_graph::edge_weight(int x, int y)
{
  node *node_x, *node_y;
  
  node_x = &(indexed_nodes[x]);
  node_y = &(indexed_nodes[y]);
  
  node::out_edges_iterator it = (*node_x).out_edges_begin();
  node::out_edges_iterator end = (*node_x).out_edges_end();
  
  for( ; it != end; ++it)
    if( (*it).target() == *node_y){
      //cout << "found edge ("<< *node_x<<", "<<*node_y<<"), weight ";//debug
      //cout <<weight[*it] << endl;;//debug
      return weight[*it];
    }
  
  return NoSuchEdge;
}

///////////////////////////////////////////////////////////////////////
bool weighted_graph::edge_exists(int x, int y)
{
  node *node_x, *node_y;
  
  node_x = &(indexed_nodes[x]);
  node_y = &(indexed_nodes[y]);
  
  node::out_edges_iterator it = (*node_x).out_edges_begin();
  node::out_edges_iterator end = (*node_x).out_edges_end();
  
  for( ; it != end; ++it)
    if( (*it).target() == *node_y) return true;
  
  return false;
}

///////////////////////////////////////////////////////////////////////
int weighted_graph::index_from_node(node & x)
{
  return node_index_map[x];
}

///////////////////////////////////////////////////////////////////////
node & weighted_graph::node_from_index(int index)
{
  return indexed_nodes[index];
}
  


