/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
#ifndef MAXCUT_GRAPHS_H
#define MAXCUT_GRAPHS_H

#ifndef EdgeHandler
#define EdgeHandler

#include <GTL/graph.h>
#include <GTL/dfs.h>
#include <GTL/debug.h>
#include <cstdlib>
#include <cassert>
#include <fstream>
#include <iostream>
#include <vector.h>
#include "../errors.h"

typedef 
/**In order to correctly assign weights to edges, we maintain a structure.
 * This structure is populated after the parsing of every edge.
 * It is read, and an edge map is produced, once the graph has been wholly read.
 *
 * @param e
 * The edge.
 *
 * @param w
 * The weight.
 */
struct EdgeHandler{
	edge e;
	double w;
} edge_handler;


#define NoSuchEdge 10000000
#endif

/** The adjacency list weight graph class extends the GTL
 * graph class in order to take advantage of weight information.
 * It also provides functionality used by the graph class.
 * 
 *  @author Pavel Sumazin
 *
 *  @version 1.7
 *
 *  @see Graph
 */
/**
@author Vinhthuy Phan
@version 1.0
*/
class weighted_graph : public graph{

private:
  /// used to map edges to their weights.
  edge_map<double> weight;
  
  /// used to map nodes to their index.
  node_map<int> node_index_map;
  
  /// used to map indexes to nodes. The first node index is 1.
  node *indexed_nodes;
  
  /// An intermediate structure to contain edge, weight pairs.
  std::vector<edge_handler> weight_vector;
  
public:
  ///
  weighted_graph(const char *file_name);
  ///
  weighted_graph();
  ///
  ~weighted_graph();
  
/** An implementation of the base class' virtual function. Loads edge, weight
 *	pairs into weight_vector.
 */
  void load_edge_info_handler (edge e, GML_pair* list);
  
  /** An implementation of the base class' virtual function. Is
   * called once the entire graph has been loaded. It uses the graph information
   * to fill in the maps and indices.
   */
  void load_graph_info_handler(GML_pair* list);
  
  /// Given an edge (x,y) two node indices, returns the edge weight.
  double edge_weight(int x, int y);
  
  /// Given an edge (x,y) two node indices, returns existence.
  bool edge_exists(int x, int y);
  
  /// Given a node, returns its index.
  int index_from_node(node & x);
  
  /// Given an index returns its node.
  node & node_from_index(int index);
  ///
  static int size;
  ///
  static int get_size();

	
};
#endif
