/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

#include <iostream>
#include <fstream>
using std::cout;
using std::endl;
using std::cerr;
using std::ifstream;
using std::ios;

#include "clauses.h"

int Clause::size = 0;

//---------------------------------------------------------------------------------
int Clause::get_size()
{
  return size;
}
//---------------------------------------------------------------------------------
int Clause::num_clauses() const
{
  return num_clause;
}
//---------------------------------------------------------------------------------
Clause::Clause(char *sat_file_name)
{
  ifstream input(sat_file_name, ios::in);
  if(! input){
    cerr<<" error opening "<<sat_file_name<<endl;
    exit(0);
  }
  input >> num_var;
  input >> num_clause;
  total_weight = 0;

  int_2d_array *tmp;
  tmp = new int_2d_array(num_var + 1);
  the_var = *tmp;
  delete tmp;

  for(int i=0; i<num_clause; i++){
    int cur_clause_var;
    double cur_clause_weight;
    int_vect cur_clause;

    input >> cur_clause_var;
    input >> cur_clause_weight;
    weight.push_back(cur_clause_weight);
    total_weight += cur_clause_weight;

    for(int j=0; j<cur_clause_var; j++){
      int v, index;
      input >> v;
      cur_clause.push_back(v);

      if(v>0) index = v;
      else index = -v;

      the_var[index].push_back(i);
    }
    the_clause.push_back(cur_clause);
  }
#if 0
  display();
#endif

  size = num_var;
}

//---------------------------------------------------------------------------------
Clause::~Clause()
{
  weight.erase(weight.begin(), weight.end());
  for(int i=0; i<num_clause; i++){
    the_clause[i].erase(the_clause[i].begin(), the_clause[i].end());
  }
  the_clause.erase(the_clause.begin(), the_clause.end());
}

//---------------------------------------------------------------------------------
double Clause::get_total_weight() const
{
  return total_weight;
}
//---------------------------------------------------------------------------------
double Clause::get_weight(int i) const
{
  if(i >= num_clause){
    cerr << "error in get_weight, "<<i<<" too large\n";
    exit(0);
  }
  return weight[i];
}

//---------------------------------------------------------------------------------
// returns true if assignment ass satisfies clause i
// ass consists of the set of true variables
bool Clause::evaluate_clause(const assignment & T_vars, 
			     const assignment & F_vars,
			     bool complete_solution,
			     int i) const
{
  if(i >= num_clause){
    cerr << "error in get_weight, "<<i<<" too large\n";
    exit(0);
  }
  int j, size = (int) the_clause[i].size();
  for(j=0; j<size; j++){
    int index = (this->the_clause[i])[j];
    if(index < 0) index = -index;

    if(T_vars.find(index) != T_vars.end()){
      if((this->the_clause[i])[j] > 0) return true;
    }
    else {
      if(complete_solution){
	if((this->the_clause[i])[j] < 0) return true;
      }
      else {
	if(F_vars.find(index) != F_vars.end() && (this->the_clause[i])[j] < 0)
	  return true;
      }
    }
  }
  return false;
}
//---------------------------------------------------------------------------------
// returns the weight of all clauses containing variable i, satisfied by ass
double Clause::total_weight_of(const assignment & T_vars,
			       const assignment & F_vars,
			       bool complete_solution,
			       int i) const
{
  double sum_weight =0;
  int j, size=(int) the_var[i].size();
  for(j=0; j<size; j++){
    if(evaluate_clause(T_vars, F_vars, complete_solution, the_var[i][j])){
#if 0
      cout<<"clause "<<the_var[i][j]<<" satisfied with weight "<<weight[the_var[i][j]]<<endl;
#endif
      sum_weight += weight[the_var[i][j]];
    }
#if 0
    else
      cout<<"clause "<<the_var[i][j]<<" not satisfied\n";
#endif
  }
  return sum_weight;
}
//---------------------------------------------------------------------------------
void Clause::display() const
{
  int i,j, size;
  cout << num_var << " variables, "<< num_clause << " clauses.\n";
  cout << "total weight : " <<total_weight<<endl;
  cout << "clause:\n";
  for(i=0; i<num_clause; i++){
    cout<<weight[i]<<":  ";
    size = (int) the_clause[i].size();
    for(j=0; j<size; j++)
      cout<<the_clause[i][j]<<"  ";
    cout<<endl;
  }
  cout << "var:\n";
  for(i=1; i<=num_var; i++){
    size = (int) the_var[i].size();
    for(j=0; j<size; j++)
      cout<<the_var[i][j]<<"  ";
    cout<<endl;
  }
  cout<<"end display\n";
}

//---------------------------------------------------------------------------------
#if 0
main()
{
  Clause a("tmp.sat");
  a.display();

  assignment A;
  A.insert(1);

  A.erase(1);
  // this means x1, x2 are true, x3,x4 are false

  cout<<a.evaluate_clause(A,0)<<endl;
  cout<<a.evaluate_clause(A,1)<<endl;
  cout<<a.evaluate_clause(A,2)<<endl;
  cout<<endl;
  cout<<a.total_weight_of(A,1)<<endl;
  cout<<a.total_weight_of(A,2)<<endl;
  cout<<a.total_weight_of(A,3)<<endl;
  cout<<a.total_weight_of(A,4)<<endl;
}
#endif
