/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

#ifndef CLAUSES_H
#define CLAUSES_H

#include <vector>
#include <algorithm>
#include <set>

using std::vector;
using std::set;

typedef  set< int > assignment;
typedef  vector< int > int_vect;
typedef  vector< double > clause_weight;
typedef  vector< int_vect > int_2d_array;

/// ---------------------------------------------------------------------------------
/**
@author Vinhthuy Phan
@version 1.0
*/
class Clause {
private:
  ///
  int num_var;
  ///
  int num_clause;
  ///
  double total_weight;
  ///
  clause_weight weight;

  /// each clause contains a valued set of variables
  /// clauses start from 0
  int_2d_array the_clause;

  /// each variable contains a list of clauses to which it belongs
  /// clauses indexed from 0
  /// variables are indexed from 1 to n
  int_2d_array the_var;

public:
  ///
  static int size;
  ///
  static int get_size ();
  ///
  int num_clauses() const;
  ///
  Clause(char *sat_file_name);
  ///
  ~Clause();
  ///
  double get_weight(int i) const;
  ///
  double get_total_weight() const;

  /// returns true if assignment ass satisfies clause i
  bool evaluate_clause(const assignment & T_vars, const assignment & F_vars,
		       bool complete_solution, int i) const;

  /// returns the weight of all clauses containing ariable i, satisfied by ass
  double total_weight_of(const assignment & T_vars, const assignment & F_vars,
			 bool complete_solution, int i) const;
  ///
  void display() const;
};

#endif
