/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#include <iostream>
#include <set>
#include <algorithm>

using std::cout;
using std::endl;
using std::set;
typedef  set< int > assignment;

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

//--------------------------------------------------------------------------------


double ObjectiveFunction::cost(Subset & sol)
{
  assignment * T_vars, * F_vars;
  assignment dummy;
  T_vars = sol.get_subset();
  if(complete_solution_flag) F_vars = &dummy;
  else F_vars = sol.get_subset_complement();

  // now compute the cost by going through all clauses
  double score = 0;
  for(int i=0; i<search_space->num_clauses(); i++){
    if( search_space->evaluate_clause(*T_vars, *F_vars, complete_solution_flag, i) ){
      score += search_space->get_weight(i);
    }
  }

  // the system minimizes but we want to maximize the weight
  return search_space->get_total_weight() - score;
}

//--------------------------------------------------------------------------------
double ObjectiveFunction::extend_cost(Subset & sol, const mutation_element & mut_el)
{
  assignment * T_vars, * F_vars;
  assignment dummy;
  T_vars = sol.get_subset();
  if(complete_solution_flag) F_vars = &dummy;
  else F_vars = sol.get_subset_complement();

  int pos = mut_el.first;
  int new_ele = mut_el.second;
  double new_cost, old_cost;
  old_cost = search_space->total_weight_of(*T_vars, *F_vars, false, new_ele);

  if(pos==1){
    T_vars->insert(new_ele);
    new_cost = search_space->total_weight_of(*T_vars, *F_vars, false, new_ele);
    T_vars->erase(new_ele);
  }
  else{
    F_vars->insert(new_ele);
    new_cost = search_space->total_weight_of(*T_vars, *F_vars, false, new_ele);
    F_vars->erase(new_ele);
  }
  double delta = old_cost - new_cost;  // Maximization
  
  return delta;
}
//--------------------------------------------------------------------------------
// return the change in cost of mutating sol
//--------------------------------------------------------------------------------
// this is the delta_swap, sol is actually old solution
// this returns the delta-cost of (sol, swap(sol, mut_element))

double ObjectiveFunction::delta_cost(Subset & sol, const mutation_element & mut_el)
{
  assignment * T_vars, * F_vars;
  assignment dummy;
  T_vars = sol.get_subset();
  if(complete_solution_flag) F_vars = &dummy;
  else F_vars = sol.get_subset_complement();

  int swap_ele = mut_el.first;
  double new_cost, old_cost;
  old_cost = search_space->total_weight_of(*T_vars, *F_vars, complete_solution_flag, swap_ele);

  // here we have to know the sematics of mut_el    
  if(sol.contains(swap_ele)){
    T_vars->erase(swap_ele);// this variable was true.  swapping out means set it to false
    F_vars->insert(swap_ele);
    new_cost = search_space->total_weight_of(*T_vars, *F_vars, complete_solution_flag, swap_ele);

    //*** UNDO ***
    T_vars->insert(swap_ele);
    F_vars->erase(swap_ele);
  }
  else{
    T_vars->insert(swap_ele);// this variable was false.  swapping out means set it to true
    F_vars->erase(swap_ele);
    new_cost = search_space->total_weight_of(*T_vars, *F_vars, complete_solution_flag, swap_ele);

    //*** UNDO ***
    T_vars->erase(swap_ele);
    F_vars->insert(swap_ele);
  }
  double delta_cost = old_cost - new_cost;  // Maximization
  
  //  cout<<swap_ele<<"  "<<old_cost<<" "<<new_cost<<" "<<delta_cost<<endl;
  return delta_cost;
}

//--------------------------------------------------------------------------------
  
double ObjectiveFunction::correctness(Subset & sol)
{
  return 0;
}

  
double ObjectiveFunction::delta_correctness(Subset & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

  
double ObjectiveFunction::extend_correctness(Subset & sol,
					     const mutation_element &mut_el)
{
  return 0;
}
//--------------------------------------------------------------------------------
double ObjectiveFunction::true_cost(Subset & sol)
{
  return cost(sol);
}
  
double ObjectiveFunction::true_correctness(Subset & sol)
{
  return correctness(sol);
}
//--------------------------------------------------------------------------------
