/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
//--------------------------------------------------------------------------------
//---- test case: Shortest Common Superstring
//--------------------------------------------------------------------------------

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

int min(int x, int y)
{
	if (x < y) return x;
	return y;
}
//--------------------------------------------------------------------------------
//returns the overlap - largest suffix of x that is a perfix of y.
inline int overlap(string & x, string & y)
{
  int x_size = (int)x.size();
  int min_size = min(x_size, (int)y.size());
  if(x_size == min_size){
    for(int i=0; i<min_size; ++i)
      if(x.substr(i, min_size-i) == y.substr(0, min_size-i)) return min_size-i;
  }
  else{
    int size_diff = x_size - min_size;
    for(int i=0; i<min_size; ++i)
      if(x.substr(size_diff + i, min_size - i) == y.substr(0, min_size - i))
	return min_size-i;
  }
  
  return 0;
}

//--------------------------------------------------------------------------------
double ObjectiveFunction::cost(Permutation &sol)
{
  int str_overlap, first, last, i, next_i, next;
  double total_length = 0;
  first = sol.first_index();
  last = sol.last_index();
  total_length = (*search_space)[sol[first]].size();
  for(i=first; i<last; i=next_i){
    next_i = sol.next_index(i);
    next = sol[next_i];
    str_overlap = overlap((*search_space)[sol[i]], (*search_space)[next]);
    total_length += (*search_space)[next].size() - str_overlap;
  }
  return total_length;
}


//--------------------------------------------------------------------------------
  
double ObjectiveFunction::extend_cost(Permutation & sol,
				      const mutation_element &mut_el)
{
  int pos = mut_el.first;
  int new_ele = mut_el.second;

  if(pos == sol.first_index()){
    return (*search_space)[new_ele].size() -
      overlap((*search_space)[new_ele], (*search_space)[sol[sol.first_index()]]);
  }
  else if(pos == sol.last_index() + 1){
    return (*search_space)[new_ele].size() -
      overlap((*search_space)[sol[sol.last_index()]], (*search_space)[new_ele]);
  }

  double length = 0;
  int pre_ele = sol[sol.previous_index(pos)];

  length += ((*search_space)[new_ele].size() -
	     overlap((*search_space)[pre_ele], (*search_space)[new_ele]));
  length += ((*search_space)[sol[pos]].size() -
	     overlap((*search_space)[new_ele], (*search_space)[sol[pos]]));

  length -= ((*search_space)[sol[pos]].size() -
	     overlap((*search_space)[pre_ele], (*search_space)[sol[pos]]));
  return length;
}

//--------------------------------------------------------------------------------
double ObjectiveFunction::delta_cost(Permutation & sol,
				     const mutation_element & mut_el)
{
  int overlap_before=0, overlap_after=0;
  int first = mut_el.first;
  int second = mut_el.second;
  int first_element = sol[first];
  int second_element = sol[second];
  int before_first =  sol[sol.previous_index(first)];
  int after_first = sol[sol.next_index(first)];
  int before_second =  sol[sol.previous_index(second)];
  int after_second = sol[sol.next_index(second)];
  
  if(first != sol.first_index()){
    overlap_before = overlap((*search_space)[before_first], (*search_space)[first_element]);
    overlap_after = overlap((*search_space)[before_first], (*search_space)[second_element]);
  }
  if(second != sol.last_index()){
    overlap_before += overlap((*search_space)[second_element], (*search_space)[after_second]);
    overlap_after += overlap((*search_space)[first_element], (*search_space)[after_second]);
  }
  if(second == sol.next_index(first)){//adjacent
    overlap_before += overlap((*search_space)[first_element],(*search_space)[second_element]);
    overlap_after += overlap((*search_space)[second_element], (*search_space)[first_element]);
  }
  else {//not adjacent
    overlap_before += overlap((*search_space)[first_element], (*search_space)[after_first]);
    overlap_before += overlap((*search_space)[before_second],(*search_space)[second_element]);
    overlap_after += overlap((*search_space)[second_element], (*search_space)[after_first]);
    overlap_after += overlap((*search_space)[before_second], (*search_space)[first_element]);
  }
  
  return (overlap_before  - overlap_after);
}

//--------------------------------------------------------------------------------
double ObjectiveFunction::correctness(Permutation & sol)
{
  return 0;
}

double ObjectiveFunction::delta_correctness(Permutation & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

double ObjectiveFunction::extend_correctness(Permutation & sol,
					     const mutation_element &mut_el)
{
  return 0;
}
//--------------------------------------------------------------------------------
double ObjectiveFunction::true_cost(Permutation & sol)
{
  return cost(sol);
}
  
double ObjectiveFunction::true_correctness(Permutation & sol)
{
  return correctness(sol);
}

//--------------------------------------------------------------------------------
