/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
//--------------------------------------------------------------------------------
//---- test case: TSP on complete graphs
//--------------------------------------------------------------------------------

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = false;

const bool ObjectiveFunction::deltas_defined = true;

//--------------------------------------------------------------------------------

  
double ObjectiveFunction::cost(CircularPermutation & sol)
{
  int current, next, size = sol.get_permutation_size();
  double weight=0;

  //  cout<<"size = "<<size<<" : ";
  current = sol.first_index();
  for(int i=0; i<size; i++){
    next = sol.next_index(current);
    //    cout <<"("<<sol[current]<<", "<<sol[next]<<"):"<<search_space->edge_weight(sol[current], sol[next]) <<" + ";
    weight += search_space->edge_weight(sol[current], sol[next]);
    current = next;
  }
  //  cout<<" 0 = "<<weight<<endl;
  return weight;
}

//--------------------------------------------------------------------------------
// return the change in cost of mutating sol
//--------------------------------------------------------------------------------
  
  
double ObjectiveFunction::delta_cost(CircularPermutation & sol,
				     const mutation_element &mut_el)
{
  double delta_weight = 0; //the weight difference.
  int first;//The first element in the swap.
  int second;//The second element in the swap.
  int before_first; //the element before the first index swapped.
  int after_second; //the element after the second index swapped.
  int after_first;//The element after the first swapped index
  int before_second;//The element before the second swapped index
  
  int fi = mut_el.first, si = mut_el.second;
  first = sol[fi];
  before_first = sol[sol.previous_index(fi)];
  after_first = sol[sol.next_index(fi)];

  second = sol[si];
  after_second = sol[sol.next_index(si)];
  before_second = sol[sol.previous_index(si)];
  
  if(first == before_second){
    delta_weight -= search_space->edge_weight(before_first, first);
    delta_weight -= search_space->edge_weight(first, second);
    delta_weight -= search_space->edge_weight(second, after_second);

    delta_weight += search_space->edge_weight(before_first, second);
    delta_weight += search_space->edge_weight(second, first);
    delta_weight += search_space->edge_weight(first, after_second);
  }
  else if(second == before_first){
    delta_weight -= search_space->edge_weight(before_second, second);
    delta_weight -= search_space->edge_weight(second, first);
    delta_weight -= search_space->edge_weight(first, after_first);

    delta_weight += search_space->edge_weight(before_second, first);
    delta_weight += search_space->edge_weight(first, second);
    delta_weight += search_space->edge_weight(second, after_first);
  }
  else {
    delta_weight -= search_space->edge_weight(before_first, first);
    delta_weight -= search_space->edge_weight(first, after_first);
    delta_weight -= search_space->edge_weight(before_second, second);
    delta_weight -= search_space->edge_weight(second, after_second);

    delta_weight += search_space->edge_weight(before_first, second);
    delta_weight += search_space->edge_weight(second, after_first);
    delta_weight += search_space->edge_weight(before_second, first);
    delta_weight += search_space->edge_weight(first, after_second);
  }

  return delta_weight;
}

//--------------------------------------------------------------------------------
  
  
double ObjectiveFunction::extend_cost(CircularPermutation & sol,
				      const mutation_element &mut_el)
{
  double weight=0;
  int pos = mut_el.first, prev_pos = sol.previous_index(mut_el.first);
  int new_ele = mut_el.second;

  if(pos > sol.last_index()) pos = sol.first_index();

  weight = search_space->edge_weight(sol[prev_pos], new_ele) 
    + search_space->edge_weight(new_ele, sol[pos]) 
    - search_space->edge_weight(sol[prev_pos], sol[pos]);

#if 0
  cout<<"#extend_cost: "<<sol.solution_size()<<" "<<pos<<"\t"<<sol[prev_pos]<<" "<<new_ele<<" "<<sol[pos]<<endl;
  cout<<"("<<sol[prev_pos]<<", "<<new_ele<<"):"<<search_space->edge_weight(sol[prev_pos], new_ele) <<" + "
      <<"("<<new_ele<<", "<<sol[pos]<<"):"<<search_space->edge_weight(new_ele, sol[pos]) <<" - "
      <<"("<<sol[prev_pos]<<", "<<sol[pos]<<"):"<<search_space->edge_weight(sol[prev_pos], sol[pos]) << " = "<<weight << endl;
#endif

  return weight;
}

//--------------------------------------------------------------------------------

  
double ObjectiveFunction::correctness(CircularPermutation & sol)
{
  return 0;
}

  
double ObjectiveFunction::delta_correctness(CircularPermutation & sol,
					    const mutation_element &mut_el)
{
  return 0;
}

  
double ObjectiveFunction::extend_correctness(CircularPermutation & sol,
					     const mutation_element &mut_el)
{
  return 0;
}

//--------------------------------------------------------------------------------
  
double ObjectiveFunction::true_correctness(CircularPermutation & solution)
{
  return correctness(solution);
}

double ObjectiveFunction::true_cost(CircularPermutation & solution)
{
  return cost(solution);
}
//--------------------------------------------------------------------------------
