/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
#include "tsp_graphs.h"

using std::ofstream;
using std::cout;
using std::endl;

int Graph::size = 0;

int Graph::get_size(){ return size; }

//--------------------------------------------------------------------
//---- GRAPH
//--------------------------------------------------------------------
Graph::Graph(char *GML_file_name)
{
  if(G.load(GML_file_name).err_num != GML_OK){
    cout << "could not open file " << GML_file_name << " to read graph\n";
    exit(0);
  }

  num_of_nodes = G.number_of_nodes();
  num_of_edges = G.number_of_edges();
  
  size = num_of_nodes;
}
//--------------------------------------------------------------------
Graph::~Graph(){  }

//--------------------------------------------------------------------
void Graph::display()
{
  cout << num_of_nodes << " nodes, "<< num_of_edges << " edges\n";
  cout << G << endl;
}

//--------------------------------------------------------------------
void Graph::display(const char *out_file){  G.save(out_file);  }
//--------------------------------------------------------------------

int Graph::get_num_of_edges(){ return num_of_edges; }

//--------------------------------------------------------------------

int Graph::index_from_node(node & x){	return G.index_from_node(x);  }

//--------------------------------------------------------------------
node & Graph::node_from_index(int index){  return G.node_from_index(index);  }

//--------------------------------------------------------------------
// indexed_nodes starts from 0
double Graph::edge_weight(int x, int y)
{
  return G.edge_weight(x,y);
}
//--------------------------------------------------------------------

bool Graph::edge_exists(int x, int y){  return G.edge_exists(x,y);  }

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------
//---- WEIGHTED GRAPH
//--------------------------------------------------------------------
//--------------------------------------------------------------------
weighted_graph::~weighted_graph()
{
  for(int i=1; i<=this->number_of_nodes();++i)
    delete [] this->adjacency_matrix[i];
  delete [] adjacency_matrix;
  delete [] indexed_nodes;
}

//--------------------------------------------------------------------

void weighted_graph::load_node_info_handler (node v, GML_pair* list)
{
	node_index_map[v] = node_i;
	++node_i;
}

//--------------------------------------------------------------------

void weighted_graph::load_edge_info_handler (edge e, GML_pair* list)
{
	if(list == NULL) error_call(missing_weights);

  edge_handler eh;
  
	if(list->kind == GML_DOUBLE) eh.w = list->value.floating;
	else if(list->kind == GML_INT) eh.w = list->value.integer;
	else error_call(bad_weights);
  
  eh.e = e;

  weight_vector.push_back(eh);
}

//--------------------------------------------------------------------

void weighted_graph::load_graph_info_handler(GML_pair* list){


  int number_of_nodes = this->number_of_nodes() +1;
  indexed_nodes = new node [number_of_nodes];

  graph::node_iterator end = this->nodes_end();
	graph::node_iterator it = this->nodes_begin();

  for(; it != end; ++it){
		node_index_map[*it] = number_of_nodes - node_index_map[*it] ;
	}

  for(it = this->nodes_begin(); it != end; ++it){
    indexed_nodes[node_index_map[*it]] = *it;
  }
	

	node source, target;

	adjacency_matrix = new (float*)[number_of_nodes];
	for(int i=1; i<number_of_nodes;++i)
		adjacency_matrix[i] = new float [number_of_nodes];
	
	for(int i=1; i<number_of_nodes;++i)
		for(int j=1; j<number_of_nodes;++j)
			adjacency_matrix[i][j] = NoSuchEdge;

	std::vector<edge_handler>::const_iterator weight_iter = weight_vector.begin();
	std::vector<edge_handler>::const_iterator weight_iter_end =
		weight_vector.end();
	
	for( ; weight_iter != weight_iter_end; ++weight_iter){
		source = weight_iter->e.source();
		target = weight_iter->e.target();

#if 0
		cout << index_from_node(source) << " " << index_from_node(target)
			<< " is " << weight_iter->w << endl;
#endif

		adjacency_matrix[index_from_node(source)][index_from_node(target)]
			= weight_iter->w;
	}

}

//--------------------------------------------------------------------

float weighted_graph::edge_weight(int x, int y)
{
	return adjacency_matrix[x][y];
}

//--------------------------------------------------------------------

bool weighted_graph::edge_exists(int x, int y)
{
  if(adjacency_matrix[x][y] != NoSuchEdge) return true;
	else return false;
}

//--------------------------------------------------------------------

int weighted_graph::index_from_node(node & x)
{
  return node_index_map[x];
}

//--------------------------------------------------------------------

node & weighted_graph::node_from_index(int index)
{
  return indexed_nodes[index];
}

//--------------------------------------------------------------------
