/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/

#include "../objective_function.h"

const bool ObjectiveFunction::correctness_defined = true;

const bool ObjectiveFunction::deltas_defined = true;

//--------------------------------------------------------------------------------

double ObjectiveFunction::cost(Subset & sol)
{
  double cost;
  cost = (double)sol.get_subset_size();
  return cost;
}

//--------------------------------------------------------------------------------

double ObjectiveFunction::extend_cost(Subset & sol,
				     const mutation_element & mut_el)
{
  int pos = mut_el.first;
  if(pos == 1) return 1;
  return 0;
}

//--------------------------------------------------------------------------------

double ObjectiveFunction::delta_cost(Subset & sol,
				     const mutation_element & mut_el)
{
  if(sol.contains(mut_el.first)) return -1;
  else return 1;
}

//--------------------------------------------------------------------------------


double ObjectiveFunction::correctness(Subset & sol)
{
  edge_set * cur_E;
  long int first_v = search_space->first_vertex(), last_v = search_space->last_vertex();
  long int i, uncovered = 0;
  for(i=first_v; i<=last_v; i++){
    if(sol.having_element(i) && ! sol.contains(i)){
      cur_E = search_space->get_edges(i);
      edge_set::iterator f = cur_E->begin(), l = cur_E->end(), it;
      for(it=f; it!=l; it++){
	if(sol.having_element(it->vertex) && ! sol.contains(it->vertex)){
	  uncovered++;
	}
      }
    }
  }
  // in an undirected graph, each uncovered edge is counted twice
  //  cout<<"#uncovered is "<<uncovered<<endl<<sol<<endl<<endl;
  return 0.5*uncovered;
}

//--------------------------------------------------------------------------------

double ObjectiveFunction::extend_correctness(Subset & sol,
					     const mutation_element & mut_el)
{
  int pos = mut_el.first;
  int new_ele = mut_el.second;
  if(pos == 1){
    // vertex is selected, thus all of its edges are covered
    return 0;
  }
  long uncovered = 0;
  edge_set * E;
  E = search_space->get_edges(new_ele);
  edge_set::iterator f = E->begin(), l = E->end(), it;
  for(it = f; it != l; it++){
    if(sol.having_element(it->vertex) && ! sol.contains(it->vertex)) uncovered++;
  }
  return uncovered;
}

//--------------------------------------------------------------------------------


double ObjectiveFunction::delta_correctness(Subset & sol,
					    const mutation_element & mut_el)
{
  long int uncovered = 0;
  
  //Removing the vertex --> increasing the number of uncovered edges
  if(sol.contains(mut_el.first)){
    edge_set * E = search_space->get_edges(mut_el.first);
    edge_set::iterator it, f=E->begin(), l=E->end();
    for(it = f; it!=l ; it++){
      if(sol.having_element(it->vertex) && ! sol.contains(it->vertex)) uncovered++;
    }
  }
  
  //Adding the vertex --> decreasing the number of uncovered edges
  else{
    edge_set * E = search_space->get_edges(mut_el.first);
    edge_set::iterator it, f=E->begin(), l=E->end();
    for(it = f; it!=l ; it++){
      if(sol.having_element(it->vertex) && ! sol.contains(it->vertex)) uncovered--;
    }
  }
  
  return uncovered;
}
//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
double ObjectiveFunction::true_cost(Subset & sol)
{
  return cost(sol);
}
  
double ObjectiveFunction::true_correctness(Subset & sol)
{
  return correctness(sol);
}
//--------------------------------------------------------------------------------
