/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

This software uses the Graph Template Library (GTL) copyrighted by the
University of Passau, http://www.fmi.uni-passau.de/Graphlet/GTL.
*/
//--------------------------------------------------------------------
// @author: Vinhthuy Phan, 04/2002
//
// -- semi-adjacency list:
//        node access in O(1) time (array)
//        edge access in O(log n) time (STL std::set, red/black tree)  
// -- in_edge[1..n]
//        in_edge[i] : the set of all edges (i, u)
// -- out_edge[1..]
//        out_edge[i] : the set of all edges (u, i)
//
// Vertex indices start from 1.
//--------------------------------------------------------------------

#ifndef DIMACS_UNDIRECTED_GRAPH_H
#define DIMACS_UNDIRECTED_GRAPH_H


#include <iostream>
#include <fstream>
#include <set>
using std::set;

#include <string>
using std::string;

#include <sstream>
using std::istringstream;

#include <functional>

//--------------------------------------------------------------------
typedef struct my_edge
{
  int vertex;  // destination 
  double weight;  // edge weight
} Edge;

//--------------------------------------------------------------------

struct EdgeCompare : public binary_function<Edge, Edge, bool>
{
  bool operator()(const Edge & a, const Edge & b) const
  {
    return (a.vertex < b.vertex);
  }
};

typedef set< Edge, EdgeCompare > edge_set;

/// --------------------------------------------------------------------
/**
@author Vinhthuy Phan
@version 1.0
*/
class UndirectedGraph
{
  ///
  friend ostream & operator<<(ostream & out, UndirectedGraph & g);

public:
  ///
  edge_set *neighbor;
  ///
  int * vertex_weight;
  ///
  int vertices, edges;

  ///
  UndirectedGraph(char *file_name);
  ///
  ~UndirectedGraph();
  ///
  static int size;
  ///
  static int get_size () ;
  ///
  int num_vertices () const { return vertices; }
  ///
  int num_edges () const { return edges; }
  ///
  void non_exist_error(int v, char* where="") const
  {
    cerr << "UndirectedGraph::"<<where<<" "<<v << " does not exist\n";
    exit(0);
  }
  ///
  int first_vertex() const { return 1; }
  ///
  int last_vertex() const { return vertices; }


  /// ---------------- Other Methods
  ///
  int get_vertex_weight(int v) const;
  ///
  bool get_edge_weight(int source, int dest, double *weight) const;
  ///
  int degree(int v) const;
  ///
  bool edge_exist(int source, int dest) const;
  ///
  edge_set * get_edges(int v) ;
  ///
  bool remove_edge(int source, int dest) ;
  ///
  bool remove_edges_containing(int v);



};
/// --------------------------------------------------------------------

#endif
