/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

#ifndef BASIC_TYPES_H
#define BASIC_TYPES_H

#include <vector>
//#include <iostream>
#include <bitset>
#include <math.h>

using namespace::std;

#include "errors.h"

//----------------------------------------------------------------------------------------

const double MAX_DOUBLE = pow(2.0, 62.0);
const int MAX_INT = (int)(pow(2.0, 31) - 1);
const int OutOfBound = -1;

///  
enum HeuristicType {h_hill_climb, h_simulated_anneal, 
		    h_greedy, h_none, h_sa_restart, h_smart_sa,
		    h_greedy_hc, h_greedy_sa, h_greedy_sa2, h_greedy_sa3, h_g_smart_sa,
		    h_cf_sa, h_g_cf_sa, h_rate_sa, h_dynamic_sa, h_dynamic_sa2,
                    h_greedyK2, h_combined };

///  
enum Status {stat_cont, stat_report, stat_terminate, stat_check_progress};
///  
enum Monitoring_mode { search_mode, non_search_mode };
///  
//enum NeighborOpType {TwoOp, Swap, CrossOver, NoOp};
enum NeighborOpType {Swap, NoOp};
///  
typedef bitset<4> MonitorStat;

//----------------------------------------------------------------------------------------
// INLINES
//----------------------------------------------------------------------------------------
template <class T>
inline T& min(T& x, T& y){  return x < y ? x : y;  }

template <class T>
inline T& max(T& x, T& y){  return x > y ? x : y;  }

//----------------------------------------------------------------------------------------
// MUTATION-ELEMENT
//----------------------------------------------------------------------------------------

/**
@author Vinhthuy Phan
@version 1.0
*/
class mutation_element {
public:
  ///  
  NeighborOpType operation;
  ///  
  int first;
  ///  
  int second;
  ///  
  mutation_element(const mutation_element & ele){
    operation = ele.operation;
    first = ele.first;
    second = ele.second;
  }

  ///  
  void set_mutation(int f, int s)
  {
    first = f; second = s;
  }

  ///  
  mutation_element(NeighborOpType op = NoOp, int f = -1, int s = -1)
  {
    set_mutation(f, s);
  }

  ///  
  mutation_element & operator=(const mutation_element & m)
  {
    if(this != &m){
      operation = m.operation;
      first = m.first;
      second = m.second;
    }
    return *this;
  }
};


typedef std::vector<mutation_element> mutation_list;


#endif
