/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#include "combinatorial_object.h"
#include <cstdlib>
#include <iostream>
using std::cout;
using std::endl;

double CombinatorialObject::constructor_count1 = 0;
double CombinatorialObject::constructor_count2 = 0;
double CombinatorialObject::constructor_count3 = 0;
double CombinatorialObject::constructor_count4 = 0;
double CombinatorialObject::destructor_count = 0;

int CombinatorialObject::MAX_SIZE  = 0;

//--------------------------------------------------------------------------------
CombinatorialObject::CombinatorialObject()
{
  constructor_count1 += 1.0;
  refCount = 0;
  num_elements = 0;
  element = element_index = NULL;
}
  
//--------------------------------------------------------------------------------

CombinatorialObject::CombinatorialObject(int n)
{
  constructor_count2 += 1.0;
  refCount = 0;
  num_elements = n;

  element = new int [MAX_SIZE + 1];
  if(element == NULL){
    cout<<"CombinatorialObject(int n): out of mem\n";
    exit(0);
  }
  element_index = new int [MAX_SIZE + 1];
  if(element_index == NULL){
    cout<<"CombinatorialObject(int n): out of mem\n";
    exit(0);
  }
  for(int i=1; i<=MAX_SIZE; i++) element[i] = element_index[i] = i;

  for(int i=1; i<MAX_SIZE; i++){
    int j = i + (rand() % (MAX_SIZE - i + 1));
    int first = element[i], second = element[j], tmp;

    tmp = element[i];
    element[i] = element[j];
    element[j] = tmp;

    tmp = element_index[first];
    element_index[first] = element_index[second];
    element_index[second] = tmp;
  }
  for(int i=n+1; i<=MAX_SIZE; i++){
    element_index[element[i]] = -1;
    element[i] = -1;
  }
}

//--------------------------------------------------------------------------------
CombinatorialObject::CombinatorialObject(const CombinatorialObject & it)
{
  constructor_count3 += 1.0;
  refCount = 0;
  num_elements = it.num_elements;
  element = new int [MAX_SIZE + 1];
  if(element == NULL){
    cout<<"CombinatorialObject(it): out of mem\n";
    exit(0);
  }
  element_index = new int [MAX_SIZE + 1];
  if(element_index == NULL){
    cout<<"CombinatorialObject(it): out of mem\n";
    exit(0);
  }
  for(int i=1; i<=MAX_SIZE; ++i){
    element[i] = it.element[i];
    element_index[i] = it.element_index[i];
  }
}

//--------------------------------------------------------------------------------
//--- insert ele AT position pos

CombinatorialObject::CombinatorialObject(const CombinatorialObject & it, int pos, int ele)
{
  //cout<<"inserting "<<ele<<" into position "<<pos<<endl;
  constructor_count4 += 1.0;
  refCount = 0;
  num_elements = it.num_elements + 1;

  int i;
  element = new int [MAX_SIZE + 1];
  if(element == NULL){
    cout<<"CombinatorialObject(it, pos, ele): out of mem\n";
    exit(0);
  }
  element_index = new int [MAX_SIZE + 1];
  if(element_index == NULL){
    cout<<"CombinatorialObject(it, pos, ele): out of mem\n";
    exit(0);
  }

  for(i=1; i<=MAX_SIZE; i++) element_index[i] = element[i] = -1;

  //  if(&it != NULL){
  for(i=1; i<pos; i++){
    element[i] = it.element[i];
    element_index[element[i]] = i;
  }
  element[pos] = ele;
  element_index[ele] = pos;
  for(i=pos+1; i<=num_elements; i++){
    element[i] = it.element[i-1];
    element_index[element[i]] = i;
  }

  //}
  //  else {
  //    element[pos] = ele;
  //    element_index[ele] = pos;
  //  }
}

//--------------------------------------------------------------------------------
CombinatorialObject::~CombinatorialObject() 
{ 
  destructor_count += 1.0;
  if(element != NULL) delete [] element;
  if(element_index != NULL) delete [] element_index;
}


//--------------------------------------------------------------------------------
int CombinatorialObject::first_index() const {  return 1; }

//--------------------------------------------------------------------------------
int CombinatorialObject::last_index() const {  return num_elements; }

//--------------------------------------------------------------------------------
int CombinatorialObject::next_index(int i) const
{
  //  if(i==num_elements) return OutOfBound;
  return i+1;
}

//--------------------------------------------------------------------------------
int CombinatorialObject::previous_index(int i) const
{
  //  if(i==1) return OutOfBound;
  return i-1;
}

//--------------------------------------------------------------------------------
int CombinatorialObject::index_of(int ele) const
{
  return element_index[ele];
}

//--------------------------------------------------------------------------------
int CombinatorialObject::operator[](int i) const 
{
  return element[i];
}

//--------------------------------------------------------------------------------
bool CombinatorialObject::having_element(int ele) const
{
  return (element_index[ele] != -1);
}

//--------------------------------------------------------------------------
ostream &operator<<( ostream & out, const CombinatorialObject & It)
{
  if(It.no_content_flag) return out;

  const int MaxLineLength = 20;
  int line_length=0, p;
  
  out <<"[ \n";
  for(p=It.first_index(); p!=It.last_index(); ++line_length){
    out << It[p] << ' ';
    if(line_length >= MaxLineLength) {
      out << endl;
      line_length = 0;
    }
    p = It.next_index(p);
  }
  out << It[p] << "\n]";
  return out;
}
//--------------------------------------------------------------------------
