/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#ifndef COMBINATORIAL_OBJECT_H
#define COMBINATORIAL_OBJECT_H

//#include <iostream>
//#include <fstream>
#include "basic_types.h"

/**
the base class of all objects.
@author Vinhthuy Phan
@version 1.0
*/
class CombinatorialObject {
  friend ostream &operator<<( ostream & out, const CombinatorialObject & It);

public:
  ///
  const static bool no_content_flag = false;

  ///
  static double constructor_count1;
  ///
  static double constructor_count2;
  ///
  static double constructor_count3;
  ///
  static double constructor_count4;
  ///
  static double destructor_count;

  ///-- Maximum possible size
  static int MAX_SIZE;

  ///-- current size of solution
  int num_elements;

  /** remembers the number of solutions referenced to it.  Therefore,
      it is manipulated ONLY by Solution<T> */
  int refCount;
  ///
  int *element;
  ///
  int *element_index;

  ///
  CombinatorialObject();

  ///
  CombinatorialObject(int n);

  ///
  CombinatorialObject(const CombinatorialObject & it);

  ///
  CombinatorialObject(const CombinatorialObject & it, int pos, int ele);

  ///
  virtual ~CombinatorialObject();

  ///
  int get_num_elements() const { return num_elements; }

  ///
  int get_refCount() const { return refCount; }
  
  ///
  void decre_refCount() { 
    refCount--;
    //    cout<<"decre_refCount of "<<this<<", value is "<<refCount<<endl;
    //    cout.flush()<<endl;
  }
  
  ///
  void incre_refCount() {
    refCount++;
    //    cout<<"incre_refCount of "<<this<<", value is "<<refCount<<endl;
    //    cout.flush()<<endl;
  }
  
  /// -- for ordered objects ---
  int operator[](int i) const;
  ///
  int first_index() const;
  ///
  int last_index() const;
  ///
  int next_index(int i) const;
  ///
  int previous_index(int i) const;
  ///
  int index_of(int e) const;

  ///
  bool having_element(int ele) const;

  ///
  virtual int get_size() const { return num_elements; }
};

//-------------------------------------------------------------------------------

#endif
