/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
/*
 *  for help, run without any option (search)
 */
//----------------------------------------------------------------------------------------
#include <iostream>
using std::cout;
using std::endl;

#include "global.h"
#include "solution.h"
#include "engine.h"

#include "basic_types.h"
#include "objective_function.h"
#include "shell_input.h"
#include "problem_defs.h"

InputType * ObjectiveFunction::search_space = NULL;
int ObjectiveFunction::solution_size = 0;
bool ObjectiveFunction::complete_solution_flag = true;

//----------------------------------------------------------------------------------------
template <class T>
class Run {
public:
  void operator ()()
  {
    //---- Global initialization must be done first (after ShellInput)
    Global<T>::initialize(ShellInput::running_time,
			  ShellInput::neighborhood_op,
			  ShellInput::output,
			  ShellInput::input_file);
    
    //---- Search initialization
    Engine<T> v6(ShellInput::heuristic,
		 ShellInput::report_interval,
		 ShellInput::check_progress_interval);
    
    
    switch(ShellInput::heuristic){
    case h_greedy_hc:
      v6.hc_greedy_hc(); break;
    case h_greedy_sa: 
      v6.greedy_sa(); break;
    case h_greedy_sa2: 
      v6.greedy_sa2(); break;
    case h_greedy_sa3: 
      v6.greedy_sa3(); break;
    case h_combined:
      v6.combined_search(); break;
    default:
      v6.run_loop();
    }
  }
};

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------
/**
the main entrance of DISCROPT.
@param argc
@param char *argv[]
command-line inputs.
@author Vinhthuy Phan
@version 1.0
*/
int main(int argc, char *argv[])
{
//	for(int i=0; i<argc; i++)		std::cout<<i<<"\t"<<argv[i]<<std::endl;

  ShellInput::initialize(argc, argv);

  ObjectiveFunction::initialize(ShellInput::input_file);

  CombinatorialObject::MAX_SIZE = InputType::get_size();

  //  cout<<ShellInput::running_time<<" "<<ShellInput::report_interval<<"  "<<ShellInput::check_progress_interval<<endl;

#if CIRCULARPERMUTATION
  Run<CircularPermutation> it;
#endif
#if PERMUTATION
  Run<Permutation> it;
#endif
#if SUBSET
  Run<Subset> it;
#endif
#if PARTITION
  Run<Partition> it;
#endif
#if ORDEREDPARTITION
  Run<OrderedPartition> it;
#endif


  it();

  return 0;
}
//----------------------------------------------------------------------------------------

