/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

#ifndef OBJECTIVE_FUNCTION_H
#define OBJECTIVE_FUNCTION_H

#define local_debug 0

#include <math.h>
#include "basic_types.h"
#include "problem_defs.h"

/**
connects user-defined problem to DISCROPT.
@author Vinhthuy Phan
@version 1.0
*/
class ObjectiveFunction {
protected:
  ///
  static InputType *search_space;

  ///
  const static bool correctness_defined;
  ///
  const static bool deltas_defined;
  ///
  static int solution_size;

public:
  ///
  static bool complete_solution_flag;
  ///
  static double power;
  ///
  ObjectiveFunction() {  }  
  ///
  static void initialize(char *input_file)
  {
    search_space = new InputType(input_file);
    solution_size = InputType::get_size();
  }

  ///
  static int get_size() { return solution_size; }

  ///
  const static bool correctness_is_defined()  {  return correctness_defined;  }

  ///
  const static bool deltas_are_defined() { return deltas_defined; }
  
  //------------------------------------------------------
  // these are optional methods. When defined, these default definitios are skipped.
  // Template specialization
  //------------------------------------------------------
  ///
#ifdef PERMUTATION
  static double true_cost(Permutation & solution);
  static double true_correctness(Permutation & solution);
  static double cost(Permutation & solution) ;
  static double correctness(Permutation & solution);
  static double delta_cost(Permutation & solution, const mutation_element & mut_el) ;
  static double delta_correctness(Permutation & solution, const mutation_element & mut_el);
  static double extend_cost(Permutation & solution, const mutation_element & mut_el);
  static double extend_correctness(Permutation & solution, const mutation_element & mut_el);
#endif
#ifdef CIRCULARPERMUTATION
  static double true_cost(CircularPermutation & solution);
  static double true_correctness(CircularPermutation & solution);
  static double cost(CircularPermutation & solution) ;
  static double correctness(CircularPermutation & solution);
  static double delta_cost(CircularPermutation & solution, const mutation_element & mut_el) ;
  static double delta_correctness(CircularPermutation & solution, const mutation_element & mut_el);
  static double extend_cost(CircularPermutation & solution, const mutation_element & mut_el);
  static double extend_correctness(CircularPermutation & solution, const mutation_element & mut_el);
#endif
#ifdef SUBSET
  static double true_cost(Subset & solution);
  static double true_correctness(Subset & solution);
  static double cost(Subset & solution) ;
  static double correctness(Subset & solution);
  static double delta_cost(Subset & solution, const mutation_element & mut_el) ;
  static double delta_correctness(Subset & solution, const mutation_element & mut_el);
  static double extend_cost(Subset & solution, const mutation_element & mut_el);
  static double extend_correctness(Subset & solution, const mutation_element & mut_el);
#endif
#ifdef PARTITION
  static double true_cost(Partition & solution);
  static double true_correctness(Partition & solution);
  static double cost(Partition & solution) ;
  static double correctness(Partition & solution);
  static double delta_cost(Partition & solution, const mutation_element & mut_el) ;
  static double delta_correctness(Partition & solution, const mutation_element & mut_el);
  static double extend_cost(Partition & solution, const mutation_element & mut_el);
  static double extend_correctness(Partition & solution, const mutation_element & mut_el);
#endif
#ifdef ORDEREDPARTITION
  static double true_cost(OrderedPartition & solution);
  static double true_correctness(OrderedPartition & solution);
  static double cost(OrderedPartition & solution) ;
  static double correctness(OrderedPartition & solution);
  static double delta_cost(OrderedPartition & solution, const mutation_element & mut_el) ;
  static double delta_correctness(OrderedPartition & solution, const mutation_element & mut_el);
  static double extend_cost(OrderedPartition & solution, const mutation_element & mut_el);
  static double extend_correctness(OrderedPartition & solution, const mutation_element & mut_el);
#endif

};

#endif
