/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#include <cstdlib>
#include "operator.h"

//----------------------------------------------------------------------------------------
// CIRCULAR_PERMUTATION
//----------------------------------------------------------------------------------------
#if CIRCULARPERMUTATION
template<>
void SwapOp<CircularPermutation>::gen_neighbor(const CircularPermutation &s, mutation_element *mut)
{
  int random1, random2;  
  /**** main diff with permutation ****/
#if 1
  random1 = (rand() % s.get_num_elements() - 1) + 2;
  random2 = (rand() % s.get_num_elements() - 1) + 2;
#else 
  random1 = (rand() % s.get_num_elements()) + 1;
  random2 = (rand() % s.get_num_elements()) + 1;
#endif
  while(random2 == random1) random2 = (rand() % s.get_num_elements()) + 1;
  
  mut->operation = Swap;
  if(random1<random2) mut->set_mutation(random1, random2);
  else mut->set_mutation(random2, random1);
}
//----------------------------------------------------------------------------------------
template<>
void SwapOp<CircularPermutation>::commit(CircularPermutation *s, const mutation_element & mut)
{
  int first_element, second_element, tmp_element;
  int first = mut.first, second = mut.second;

  //  first_element = s->lookup_element(s->element_of(first));
  //  second_element = s->lookup_element(s->element_of(second));
  first_element = s->perm[first];
  second_element = s->perm[second];
  
  tmp_element = s->perm[first];
  s->perm[first] = s->perm[second];
  s->perm[second] = tmp_element;
  
  tmp_element = s->inverse_perm[first_element];
  s->inverse_perm[first_element] = s->inverse_perm[second_element];
  s->inverse_perm[second_element] = tmp_element;
}
//----------------------------------------------------------------------------------------
template<>
double SwapOp<CircularPermutation>::neighborhood_size(const CircularPermutation &s) const
{
  return s.get_num_elements() * s.get_num_elements() * 0.5;
}
//----------------------------------------------------------------------------------------
#endif


#if PERMUTATION
//----------------------------------------------------------------------------------------
// PERMUTATION
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Permutation>::gen_neighbor(const Permutation &s, mutation_element *mut)
{
  int random1, random2;  
  random1 = (rand() % s.get_num_elements()) + 1 ;
  random2 = (rand() % s.get_num_elements()) + 1;
  while(random2 == random1) random2 = (rand() % s.get_num_elements()) + 1;
  
  mut->operation = Swap;
  if(random1<random2) mut->set_mutation(random1, random2);
  else mut->set_mutation(random2, random1);
}
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Permutation>::commit(Permutation *s, const mutation_element & mut)
{
  int first_element, second_element, tmp_element;
  int first = mut.first, second = mut.second;

  //  first_element = s->lookup_element(s->element_of(first));
  //  second_element = s->lookup_element(s->element_of(second));
  first_element = s->perm[first];
  second_element = s->perm[second];
  
  tmp_element = s->perm[first];
  s->perm[first] = s->perm[second];
  s->perm[second] = tmp_element;
  
  tmp_element = s->inverse_perm[first_element];
  s->inverse_perm[first_element] = s->inverse_perm[second_element];
  s->inverse_perm[second_element] = tmp_element;
}
//----------------------------------------------------------------------------------------
template<>
double SwapOp<Permutation>::neighborhood_size(const Permutation &s) const
{
  return s.get_num_elements() * s.get_num_elements() * 0.5;
}
//----------------------------------------------------------------------------------------
#endif



#if SUBSET
//----------------------------------------------------------------------------------------
// SUBSET
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Subset>::gen_neighbor(const Subset &s, mutation_element *mut)
{
  mut->operation = Swap;
  mut->set_mutation(s[(rand() % s.get_num_elements()) + 1], 1);
}
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Subset>::commit(Subset *s, const mutation_element & mut)
{
  int ele = mut.first; //;s->element[mut.first];
  if(s->element_indicator[ele] == true){
    s->subs->erase(ele);
    s->subs_compl->insert(ele);
    s->element_indicator[ele] = false;
    (s->subset_size)--;
  }
  else{
    s->subs->insert(ele);
    s->subs_compl->erase(ele);
    s->element_indicator[ele] = true;
    (s->subset_size)++;
  }
}
//----------------------------------------------------------------------------------------
template<>
double SwapOp<Subset>::neighborhood_size(const Subset &s) const
{
  return s.get_num_elements();
}
//----------------------------------------------------------------------------------------
#endif


#if PARTITION
//----------------------------------------------------------------------------------------
// PARTITION
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Partition>::gen_neighbor(const Partition &s, mutation_element *mut)
{
  mut->operation = Swap;
  int rand_ele, rand_part;
  bool flag = true;
  while(flag){
    flag = false;
    rand_ele = s[(rand()%s.get_num_elements()) + 1];
    rand_part = rand()%(s.num_of_parts() + 1) + 1;
    if(rand_part == s.num_of_parts() + 1){
      // retry if a 1-element part is moved to a new part
      int tmp = s.which_part(rand_ele);
      if(s.partition_size(tmp) == 1) flag = true;
    }
    else {
      // or if an element is swaped to same part
      if(s.which_part(rand_ele) == rand_part) flag = true;
    }
  }

  mut->set_mutation(rand_ele, rand_part);
}
//----------------------------------------------------------------------------------------
template<>
void SwapOp<Partition>::commit(Partition *s, const mutation_element & mut)
{
  int ele = mut.first;
  int new_part = mut.second, old_part ;

  if(new_part > s->num_parts + 1){
    cerr << "commit, "<<mut.first<<" "<<mut.second<<"  "<<s->num_parts<<endl;
    exit(0);
  }

  old_part = s->which_part(ele);

  // 1- erase the element from old part
  (*(s->the_partition))[old_part].erase(ele);

  // 2- insert the element into the new part
  if(new_part == s->num_parts + 1){
    subset empty_set;
    (s->num_parts)++;
    s->the_partition->push_back(empty_set);
  }
  (*(s->the_partition))[new_part].insert(ele);
  s->part_index[ele] = new_part;

  // 3- if the old part is empty, move the last part to it and update values
  if((*(s->the_partition))[old_part].empty()){
    (*(s->the_partition))[old_part] = (*(s->the_partition))[s->num_parts];

    subset::const_iterator i;
    subset::const_iterator begin = (*(s->the_partition))[old_part].begin();
    subset::const_iterator end = (*(s->the_partition))[old_part].end();
    for(i=begin; i!=end; ++i)
      (s->part_index)[*i] = old_part;
    (s->the_partition)->erase(s->the_partition->end() - 1);
    (s->num_parts)--;
  }
}
//----------------------------------------------------------------------------------------
template<>
double SwapOp<Partition>::neighborhood_size(const Partition &s) const
{
  return s.num_of_parts() * s.get_num_elements();
}
//----------------------------------------------------------------------------------------
#endif


#if ORDEREDPARTITION
//----------------------------------------------------------------------------------------
// ORDERED PARTITION
//----------------------------------------------------------------------------------------
template<>
void SwapOp<OrderedPartition>::gen_neighbor(const OrderedPartition &s, mutation_element *mut)
{
  mut->operation = Swap;
  int rand_ele, rand_part_index;
  bool flag = true;

  while(flag){
    flag = false;
    rand_ele = s[(rand()%s.get_num_elements()) + 1];
    rand_part_index = rand()%(s.num_of_parts() + 1) + 1;

    //-- avoid moving to the same partition
    if(s.which_part(rand_ele) == rand_part_index) flag = true;
  }

  mut->set_mutation(rand_ele, rand_part_index);
}
//----------------------------------------------------------------------------------------
template<>
void SwapOp<OrderedPartition>::commit(OrderedPartition *s, const mutation_element & mut)
{
  int ele = mut.first;
  int new_part_index = mut.second, old_part_index;

  old_part_index = s->which_part(ele);

  // 1- erase the element from old partition
  (*(s->the_partition))[old_part_index].erase(ele);
  
  // 2- insert the element into the new partition index
  if(new_part_index == s->num_parts + 1){
    subset empty_set;
    (s->num_parts)++;
    s->the_partition->push_back(empty_set);
  }
  (*(s->the_partition))[new_part_index].insert(ele);
  s->part_index[ele] = new_part_index;

  if(old_part_index == s->num_parts && new_part_index < s->num_parts &&
     (*(s->the_partition))[old_part_index].empty())
    {
      (s->the_partition)->erase(s->the_partition->end() - 1);
      (s->num_parts)--;
    }
}
//----------------------------------------------------------------------------------------
template<>
double SwapOp<OrderedPartition>::neighborhood_size(const OrderedPartition &s) const
{
  return s.num_of_parts() * s.get_num_elements();
}
//----------------------------------------------------------------------------------------


#endif
