/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#ifndef OPERATOR_H
#define OPERATOR_H

#include "basic_types.h"
#include "combinatorial_object.h"
#include "problem_defs.h"

/**
the base class of all neighborhood structures.
@author Vinhthuy Phan
@version 1.0
*/
template <class T>
class NeighborOp {

public:
  ///
  NeighborOp () { }

  ///
  virtual ~NeighborOp () { }

  ///
  virtual void gen_neighbor(const T &s, mutation_element *mut) = 0;
  ///
  virtual void commit(T *s, const mutation_element & mut) = 0;
  ///
  virtual double neighborhood_size(const T &s) const = 0;

  ///
  virtual NeighborOpType get_type() const = 0;
};

/**
the "swap" neighborhood operator.
@author Vinhthuy Phan
@version 1.0
*/
template <class T>
class SwapOp : public NeighborOp<T> {
public:
  ///
  SwapOp() : NeighborOp<T> () { }

  ///
  void gen_neighbor(const T &s, mutation_element *mut);
  ///
  void commit(T *s, const mutation_element & mut);
  ///
  double neighborhood_size(const T &s) const;

  ///
  NeighborOpType get_type() const { return Swap; }
};

#endif
