/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
//--------------------------------------------------------------------
//
// partition.cpp: implementation of the OrderedPartition class.
//
//--------------------------------------------------------------------
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <cstdlib>
#include "ordered_partition.h"

//--------------------------------------------------------------------------
//---- Main constructor
//--------------------------------------------------------------------------
OrderedPartition::OrderedPartition() : CombinatorialObject()
{
  part_index = NULL;
  the_partition = NULL;
  num_parts = 0;
}
//--------------------------------------------------------------------------
OrderedPartition::OrderedPartition(int n) : CombinatorialObject(n)
{
  part_index = new int[MAX_SIZE+1];
  the_partition = new set_partition;
  the_partition->push_back(the_empty_part);
  the_partition->push_back(the_empty_part);

  int i;
  num_parts = 1;
  (*the_partition)[1].insert(element[1]);
  part_index[element[1]] = 1;
  for(i=2; i<=n; i++){
    // 1 <= rand_part <= num_parts+1
    int rand_part = rand()%(num_parts + 1) + 1;
    if(rand_part == num_parts + 1){
      num_parts++;
      the_partition->push_back(the_empty_part);
    }
    (*the_partition)[rand_part].insert(element[i]);
    part_index[element[i]] = rand_part;
  }
  for(i=n+1; i<=MAX_SIZE; i++) part_index[element[i]] = -1;
}
//--------------------------------------------------------------------------
//---- Copy constructor
//--------------------------------------------------------------------------
OrderedPartition::OrderedPartition(const OrderedPartition &p) : CombinatorialObject(p)
{
  num_parts = p.num_parts;
  part_index = new int[MAX_SIZE+1];
  for(int i=1; i<=MAX_SIZE; i++) part_index[i] = p.part_index[i];
  the_partition = new set_partition;
  *the_partition = *(p.the_partition);
}
//--------------------------------------------------------------------------

OrderedPartition::OrderedPartition(const OrderedPartition &p, int pos, int e) 
  : CombinatorialObject(p, p.num_elements+1, e)
{
  int n = num_elements;
  part_index = new int [n+1];
  if(&p == NULL){
    num_parts = 0;
    the_partition = new set_partition;
    the_partition->push_back(the_empty_part);
    the_partition->push_back(the_empty_part);
  }
  else {
    num_parts = p.num_parts;
    for(int i=1; i<n; i++) part_index[i] = p.part_index[i];
    the_partition = new set_partition;
    *the_partition = *(p.the_partition);
  }

  if(pos == num_parts + 1){
    num_parts++;
    the_partition->push_back(the_empty_part);
  }
  (*the_partition)[pos].insert(n);
  part_index[n] = pos;
}

//--------------------------------------------------------------------------
OrderedPartition::~OrderedPartition()
{
  if(part_index != NULL)  delete [] part_index;
  if(the_partition != NULL) delete the_partition;
}

//--------------------------------------------------------------------------
// ---- Indexing methods for partition and its inverse
//--------------------------------------------------------------------------
int OrderedPartition::first_partition() const
{
  return 1;
}

//--------------------------------------------------------------------------
int OrderedPartition::last_partition () const 
{
  return num_parts;
}

//--------------------------------------------------------------------------

subset OrderedPartition::get_partition(int index_p) const
{
  if(index_p == last_partition()+1){
    return the_empty_part;
  }
  else if(index_p < first_partition() || index_p > last_partition()){
    cerr << "OrderedPartition::partition_lookup: "<<index_p<<" is out of bound\n";
    exit(0);
  }
  return (*the_partition)[index_p];
}

//--------------------------------------------------------------------------
int OrderedPartition::which_part(int e) const
{
  if(e > num_elements) return -1;
  return part_index[e];
}

//--------------------------------------------------------------------------

int OrderedPartition::partition_size(int index_p) const
{
  if(index_p == num_parts+1) return 0;
  else if(index_p < first_partition() || index_p > last_partition()){
    cerr << "OrderedPartition::partition_lookup: "<<index_p<<" is out of bound\n";
    exit(0);
  }
  return (int) (*the_partition)[index_p].size();
}

//--------------------------------------------------------------------------

ostream &operator<<( ostream & out, const OrderedPartition & It)
{
  const int MaxLineLength = 20;
  int len;    
  out <<"(ordered_partition, [ \n";
  set_partition::iterator i, begin=It.the_partition->begin(), end=It.the_partition->end();
  subset::iterator j, j_beg, j_end;
  for(i = begin; i!=end; i++){
    j_beg = i->begin();
    j_end = i->end();
    out<<"( ";
    for(len=0, j = j_beg; j!=j_end; j++, len++){
      out << *j << ' ';
      if(len==MaxLineLength){ len=0; out << endl; }
    }      
    out << " )\n";
  }
  out << "])\n";
  return out;
}

//--------------------------------------------------------------------------
// --------- Partial Solution Support
//--------------------------------------------------------------------------

int OrderedPartition::first_position() const { return 1; }

//--------------------------------------------------------------------------

int OrderedPartition::last_position() const { return num_parts + 1; }

//--------------------------------------------------------------------------
