/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#ifndef ORDERED_PARTITION_H
#define ORDERED_PARTITION_H


#include <set>
#include <vector>
#include "basic_types.h"
#include "combinatorial_object.h"

typedef std::set<int> subset;
typedef std::vector<subset> set_partition;


/**
a type of combinatorial object; can not manipulate refCount.
@author Vinhthuy Phan
@version 1.0
*/
class OrderedPartition : public CombinatorialObject  {
  ///
  friend ostream &operator<<( ostream & out, const OrderedPartition & It);
  
public:
  /** part_index and the_partition indexed from 1, 
      stores the indices of which partition element i is in */
  int *part_index;

  /// vector of subsets each stores a set of elements
  set_partition *the_partition;
  ///
  subset the_empty_part;
  ///
  int num_parts;
  ///
  bool order_matters(){  return false; }
  ///
  OrderedPartition();
  ///
  OrderedPartition(int n);
  ///
  OrderedPartition(const OrderedPartition &p);
  ///
  OrderedPartition(const OrderedPartition &p, int pos, int ele);
  
  /// --- partial solution support -----
  int first_position() const;
  ///
  int last_position() const;

  ///
  ~OrderedPartition();
  
  ///
  int num_of_parts() const {
    return num_parts;
  }

  ///
  void display(ostream & out, int flag=-1) const;
  
  ///
  int first_partition() const;
  ///
  int last_partition() const;
  ///
  subset get_partition(int index_p) const;
  ///
  int which_part(int i) const;
  ///
  int partition_size(int index) const;

};

#endif
