/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
//--------------------------------------------------------------------
//
// partition.cpp: implementation of the Partition class.
//
//--------------------------------------------------------------------
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <cstdlib>
#include "partition.h"

//--------------------------------------------------------------------------
//---- Main constructor
//--------------------------------------------------------------------------
Partition::Partition() : CombinatorialObject()
{
  part_index = NULL;
  the_partition = NULL;
  num_parts = 0;
}
//--------------------------------------------------------------------------
Partition::Partition(int n) : CombinatorialObject(n)
{
  part_index = new int[MAX_SIZE+1];
  the_partition = new set_partition;
  the_partition->push_back(the_empty_part);
  the_partition->push_back(the_empty_part);
  
  int i;
  num_parts = 1;
  (*the_partition)[1].insert(element[1]);
  part_index[element[1]] = 1;
  for(i=2; i<=n; i++){
    // 1 <= rand_part <= num_parts+1
    int rand_part = rand()%(num_parts + 1) + 1;
    if(rand_part == num_parts + 1){
      num_parts++;
      the_partition->push_back(the_empty_part);
    }
    (*the_partition)[rand_part].insert(element[i]);
    part_index[element[i]] = rand_part;
  }
#if 0
  for(i=n+1; i<=MAX_SIZE; i++){
    cout<<element[i]<<" ";
    part_index[element[i]] = -1;
  }
  cout<<endl;
#endif

}
//--------------------------------------------------------------------------
//---- Copy constructor
//--------------------------------------------------------------------------
Partition::Partition(const Partition &p) : CombinatorialObject(p)
{
  num_parts = p.num_parts;
  part_index = new int[MAX_SIZE+1];
  for(int i=1; i<=MAX_SIZE; i++) part_index[i] = p.part_index[i];
  the_partition = new set_partition;
  *the_partition = *(p.the_partition);
  //the_partition = new set_partition(*(p.the_partition));
}
//--------------------------------------------------------------------------

Partition::Partition(const Partition &p, int pos, int e) 
  : CombinatorialObject(p, p.num_elements+1, e)
{
  //  cout<<"B "<<p.num_elements+1<<endl;

  num_parts = p.num_parts;
  part_index = new int[MAX_SIZE+1];
  for(int i=1; i<=MAX_SIZE; i++) part_index[i] = p.part_index[i];

  the_partition = new set_partition;
  *the_partition = *(p.the_partition);
  //  the_partition = new set_partition(*(p.the_partition));

  if(pos == num_parts + 1){
    num_parts++;
    the_partition->push_back(the_empty_part);
  }
  (*the_partition)[pos].insert(e);
  part_index[e] = pos;
}

//--------------------------------------------------------------------------
Partition::~Partition()
{
  //  cout<<"freeing "<<this<<endl;
  //  cout.flush()<<endl;
  if(part_index != NULL)  delete [] part_index;
  if(the_partition != NULL) delete the_partition;
}

//--------------------------------------------------------------------------
// ---- Indexing methods for partition and its inverse
//--------------------------------------------------------------------------
int Partition::first_part() const
{
  return 1;
}

//--------------------------------------------------------------------------
int Partition::last_part () const 
{
  return num_parts;
}

//--------------------------------------------------------------------------

subset Partition::get_part(int index_p) const
{
  //  cout<<"get partition of " << index_p<<endl;
  if(index_p == last_part()+1){
    return the_empty_part;
  }
  else if(index_p < first_part() || index_p > last_part()){
    cerr << "Partition::get_part: "<<index_p<<" is out of bound\n";
    cerr << "first part "<<first_part()<<", last part "<<last_part()<<endl;
    exit(0);
  }
  return (*the_partition)[index_p];
}

//--------------------------------------------------------------------------

int Partition::which_part(int e) const
{
  //  cout<<"part index of "<<e<<" is "<<part_index[e]<<endl;
  return part_index[e];
}

//--------------------------------------------------------------------------

int Partition::partition_size(int index_p) const
{
  if(index_p == last_part()+1) return 0;
  else if(index_p < first_part() || index_p > last_part()){
    cerr << "Partition::partition_size: "<<index_p<<" is out of bound\n";
    exit(0);
  }
  return (int) (*the_partition)[index_p].size();
}

//--------------------------------------------------------------------------

ostream &operator<<( ostream & out, const Partition & It)
{
  const int MaxLineLength = 20;
  int len;    
  out <<"(partition, [ \n";
  set_partition::iterator i, begin=It.the_partition->begin(), end=It.the_partition->end();
  subset::iterator j, j_beg, j_end;
  for(i = begin; i!=end; i++){
    j_beg = i->begin();
    j_end = i->end();
    out<<"( ";
    for(len=0, j = j_beg; j!=j_end; j++, len++){
      out << *j << ' ';
      if(len==MaxLineLength){ len=0; out << endl; }
    }      
    out << " )\n";
  }
  out << "])\n";
  return out;
}

//--------------------------------------------------------------------------
// --------- Partial Solution Support
//--------------------------------------------------------------------------

int Partition::first_position() const { return 1; }

//--------------------------------------------------------------------------

int Partition::last_position() const { return num_parts + 1; }

//--------------------------------------------------------------------------
