/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#ifndef PARTITION_H
#define PARTITION_H


//#include <iostream>
//#include <fstream>
#include <vector>
#include <set>
#include "basic_types.h"
#include "combinatorial_object.h"

typedef std::set<int> subset;
typedef std::vector<subset> set_partition;


/**
a type of combinatorial object; can not manipulate refCount.
@author Vinhthuy Phan
@version 1.0
*/
class Partition : public CombinatorialObject  {
  ///
  friend ostream &operator<<( ostream & out, const Partition & It);
  
public:
  /** part_index and the_partition indexed from 1, 
      stores the indices of which partition element i is in */
  int *part_index;

  /// vector of subsets each stores a set of elements
  set_partition *the_partition;
  ///
  int num_parts;
  ///
  subset the_empty_part;

  ///
  bool order_matters(){  return false; }
  ///
  Partition();
  ///
  Partition(int n);
  ///
  Partition(const Partition &p);
  ///
  Partition(const Partition &p, int pos, int ele);
  
  /// ---- partial solution support -----
  int first_position() const;
  ///
  int last_position() const;
  ///
  ~Partition();
  ///
  int num_of_parts() const {
    return num_parts;
  }
  ///
  void display(ostream & out, int flag=-1) const;
  
  ///
  int first_part() const;
  ///
  int last_part() const;
  ///
  subset get_part(int index_p) const;
  ///
  int which_part(int i) const;

  //  int element_lookup(int it) const;
  ///
  int partition_size(int index) const;

};

#endif
