/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/
#include <time.h>
#include <iostream>
using std::cout;
using std::endl;

#include <fstream>

#include "shell_input.h"
#include "combinatorial_object.h"

//----------------------------------------------------------------------------------------
// Created by Vinhthuy Phan
// User inputs initializations
//----------------------------------------------------------------------------------------


//-----------user input from command line
double ShellInput::running_time = 0;

double ShellInput::report_interval = 1;

double ShellInput::check_progress_interval = 1;

char * ShellInput::input_file = NULL;

ostream * ShellInput::output = &(std::cout);

HeuristicType ShellInput::heuristic = h_none;

NeighborOpType ShellInput::neighborhood_op = Swap;

//----------------------------------------------------------------------------------------
void ShellInput::initialize(int argc, char *argv[])
{
  if(argc==1){
    print_options(argv);
    error_call(bad_option);
  }

  bool non_random_run = false, will_report=false;

  for(int i=1; i<argc; i = i++){
    if(strcmp(argv[i], "--non-random") == 0){
      non_random_run = true;
      i++;
    }
    else if(argv[i][0] == '-'){
      switch(argv[i][1]){
      case 'i':
	input_file = argv[i+1]; i++;
	break;
      case 'o':
	output = new ofstream(argv[i+1], ios::out); i++;
	break;
      case 't':
	running_time = atof(argv[i+1]); i++;
	break;
      case 'r':
	report_interval = atof(argv[i+1]); i++;
	will_report = true;
	break;
      case 'c':
	check_progress_interval = atof(argv[i+1]); i++;
	break;
      case 'h':
	if(strcmp(argv[i+1], "hill_climb")==0)
	  heuristic = h_hill_climb;
	else if(strcmp(argv[i+1], "simulated_anneal")==0)
	  heuristic = h_simulated_anneal;
	else if(strcmp(argv[i+1], "smart_sa")==0)
	  heuristic = h_smart_sa;
	else if(strcmp(argv[i+1], "sa_restart")==0)
	  heuristic = h_sa_restart;
	else if(strcmp(argv[i+1], "greedy")==0)
	  heuristic = h_greedy;
	else if(strcmp(argv[i+1], "greedy_hc")==0)
	  heuristic = h_greedy_hc;
	else if(strcmp(argv[i+1], "greedy_sa")==0)
	  heuristic = h_greedy_sa;
	else if(strcmp(argv[i+1], "greedy_sa2")==0)
	  heuristic = h_greedy_sa2;
	else if(strcmp(argv[i+1], "greedy_sa3")==0)
	  heuristic = h_greedy_sa3;
	else if(strcmp(argv[i+1], "g_smart_sa")==0)
	  heuristic = h_g_smart_sa;
	else if(strcmp(argv[i+1], "cf_sa")==0)
	  heuristic = h_cf_sa;
	else if(strcmp(argv[i+1], "g_cf_sa")==0)
	  heuristic = h_g_cf_sa;
	else if(strcmp(argv[i+1], "rate_sa")==0)
	  heuristic = h_rate_sa;
	else if(strcmp(argv[i+1], "dynamic_sa")==0)
	  heuristic = h_dynamic_sa;
	//else if(strcmp(argv[i+1], "dynamic_sa2")==0)
	//heuristic = h_dynamic_sa2;
	else if(strcmp(argv[i+1], "combined")==0)
	  heuristic = h_combined;
	//else if(strcmp(argv[i+1], "greedyK2")==0)
	//heuristic = h_greedyK2;
	i++;
	break;
      case 'n':
	if(strcmp(argv[i+1], "swap")==0)  // also default
	  neighborhood_op = Swap;
	i++;
	break;
      default:
	print_options(argv);
	error_call(bad_option);
      }
    }
    else error_call(bad_option);
  }
  
  if(will_report == false) report_interval = running_time + 1;
  if(non_random_run == false) 
	  srand((unsigned int)time(NULL));

  if(!input_file){
    print_options(argv);
    cerr << "must provide input file\n"; exit(0);
  }
  if(heuristic==h_none){
    //    print_options(argv);
    cout << "### default heuristic envoked\n";
  }

  if(running_time==0){
    print_options(argv);
    cerr<<"\n\n\tMust provide positive running time\n";
  }    
}
//----------------------------------------------------------------------------------------
void ShellInput::print_options(char *argv[])
{
  cerr << endl;
  cerr << "Blackbox Optimization (v2.0)\n";
  cerr << "Copyright (C) 2001 Vinhthuy Phan, Steven Skiena, Pavel Sumazin, SUNY Stony Brook\n\n";
  cerr << "Usage:  " 
       << argv[0]<< " [options as specified below]" << endl << endl;
  cerr << "\t--non-random  non-stochastic search (since time-sensitive decisions\n"
       << "\t              are made based on system time which is dependent on\n"
       << "\t              other processes.  These decisions make not be identical\n"
       << "\t              for the same parameters even with non-random flag)\n";
  cerr << "\t-i  <input file>              required\n";
  cerr << "\t-o  <output file>             default is standard output\n";
  cerr << "\t-t  <running time>            required\n";
  cerr << "\t-r  <report interval>         default is non-report\n";
  cerr << "\t-c  <check-progress interval>\n" 
       << "\t            indicates often the search should check to see if\n"
       << "\t            no progress has been made.  This quantity is defaulted\n"
       << "\t            to be 1 sec, and automatically adjusted by the engine\n";
  cerr << "\t-h  <heuristic> one of the followings:\n"
       << "\t\thill_climb\n"
       << "\t\tsimulated_anneal: a multiply-decayed schedule\n"
       << "\t\tsmart_sa: a mulitply-decayed schedule,\n"
       << "\t\t          trial length adjusted\n"
       << "\t\t          local-restart if finish prematurely\n"
       << "\t\tgreedy: local-search randomized greedy\n"
       << "\t\tgreedy_hc: hill-climb, greedy followed by hill_climb\n"
       << "\t\tgreedy_sa: greedy followed by smart-sa\n"
       << "\t\tg_smart_sa: at each step of smart-sa, select annealing\n"
       << "\t\t            on best-of-k neighbors\n"
       << "\t\tcf_sa: annealing to fit an exponential curve\n"
       << "\t\tg_cf_sa: at each step of cf_sa, select annealing\n"
       << "\t\t            on best-of-k neighbors\n"
       << "\t\trate_sa: same as simulated_anneal, but temperature\n"
       << "\t\t         thresholds are selected according to given rates\n"
       << "\t\tdynamic_sa: incorporate variance of cost into annealing schedule\n"
       << "\t\tcombined: combines cf_sa, greedy, and hill_climb\n";
}
//----------------------------------------------------------------------------------------
