/*
Copyright (C) 2003 copyrighted by SUNY at Stony Brook.

DISCROPT is created by Vinhthuy Phan, Pavel Sumazin, and Steven Skiena.
The authors grant permision to use and distribute WITHOUT ANY WARRANTY
this software for non-commercial purposes, provided that this copyright 
notice is clearly shown.  Other uses are prohibited without prior written
consent.  These terms are applied to any modification of this software 
as well.  See http://www.cs.sunysb.edu/~discropt.

*/

#include <fstream>
//#include <cstdlib>
#include "subset.h"


Subset::Subset() : CombinatorialObject()
{
  element_indicator = NULL;
  subset_size = 0;
  subs_compl = subs = NULL;
}

//--------------------------------------------------------------------------
Subset::Subset(int n) : CombinatorialObject(n)
{
  int random;
  subset_size = 0;
  subs = new subset;
  subs_compl = new subset;
  element_indicator = new bool [MAX_SIZE + 1];
  for(int i = 1; i <= n; ++i){
    random = rand();
    if(random % 2){
      element_indicator[element[i]] = true;
      subset_size++;
      subs->insert(element[i]);
    }
    else{
      element_indicator[element[i]] = false;
      subs_compl->insert(element[i]);
    }
  }
}
  
//--------------------------------------------------------------------------
Subset::Subset(const Subset &p) : CombinatorialObject(p)
{ 
  subs = new subset(*(p.subs));
  subs_compl = new subset(*(p.subs_compl));
  subset_size = p.subset_size;
  element_indicator = new bool [MAX_SIZE + 1];
  for(int i=1; i<=MAX_SIZE; i++) element_indicator[i] = p.element_indicator[i];
}

//--------------------------------------------------------------------------

Subset::Subset(const Subset &p, int pos, int e) 
  : CombinatorialObject(p, p.num_elements+1, e)
{
  subs = new subset(*(p.subs));
  subs_compl = new subset(*(p.subs_compl));
  subset_size = p.subset_size;
  element_indicator = new bool [MAX_SIZE + 1];
  for(int i=1; i<=MAX_SIZE; i++) element_indicator[i] = p.element_indicator[i];
  if(pos == 1){
    element_indicator[e] = true;
    subset_size++;
    subs->insert(e);
  }
  else {
    element_indicator[e] = false;
    subs_compl->insert(e);
  }
}

//--------------------------------------------------------------------------
Subset::~Subset()
{
  if(element_indicator != NULL) delete element_indicator;
  if(subs != NULL) delete subs;
  if(subs_compl != NULL) delete subs_compl;
}

//--------------------------------------------------------------------------

bool Subset::contains(int e) const
{
  return (element_indicator[e] == true);
}

subset * Subset::get_subset() 
{
  return subs;
}

subset * Subset::get_subset_complement()
{
  return subs_compl;
}

//--------------------------------------------------------------------------
// --------- Partial Solution Support
//--------------------------------------------------------------------------

int Subset::first_position() const { return 1; }

//--------------------------------------------------------------------------

int Subset::last_position() const { return 2; }

//--------------------------------------------------------------------------
ostream &operator<<( ostream & out, const Subset & It)
{
  out <<"(subset, ";
  out << (CombinatorialObject) It << ")\n";
  return out;
}
//--------------------------------------------------------------------------

